@@ -24,20 +24,28 @@ fi
 cd build/
 
 MAKE_TASKS=1
-if [ -r /proc/cpuinfo ]; then
-	MAKE_TASKS=$(grep -c "^processor" /proc/cpuinfo)
-elif [ -x /sbin/sysctl ]; then
-	TMP_CORES=$(/sbin/sysctl -n hw.ncpu 2>/dev/null)
-	if [ "$?" = "0" ]; then
-		MAKE_TASKS=$TMP_CORES
+if [ `uname -s` = "SunOS" ]; then
+	MAKE_TASKS=$( /usr/sbin/psrinfo |wc -l )
+	MAKE=gmake
+	PATH=/usr/gnu/bin:$PATH ; export PATH
+else
+	if [ -r /proc/cpuinfo ]; then
+		MAKE_TASKS=$(grep -c "^processor" /proc/cpuinfo)
+	elif [ -x /sbin/sysctl ]; then
+		TMP_CORES=$(/sbin/sysctl -n hw.ncpu 2>/dev/null)
+		if [ "$?" = "0" ]; then
+			MAKE_TASKS=$TMP_CORES
+		fi
 	fi
+	MAKE=make
 fi
 
 if [ "$(($MAKE_TASKS < 1))" -eq 1 ]; then
 	MAKE_TASKS=1
 fi
 
-cmake -DCMAKE_INSTALL_PREFIX=${INSTALL_PREFIX} -DCMAKE_BUILD_TYPE=${BUILD_TYPE} -DINSTALL_IOP_EXPERIMENTAL=Off -DINSTALL_IOP_LEGACY=Off .. && make -j $MAKE_TASKS 
+cmake -DCMAKE_INSTALL_PREFIX=${INSTALL_PREFIX} -DCMAKE_BUILD_TYPE=${BUILD_TYPE} \
+	-DINSTALL_IOP_LEGACY=Off .. && $MAKE -j $MAKE_TASKS 
 
 if [ $? = 0 ]; then
 	echo "Darktable finished building, to actually install darktable you need to type:"
