@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2010 Henrik Andersson.
+    copyright (c) 2010-2012 Henrik Andersson.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,34 +16,28 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stdio.h>
-#include <memory.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <glib.h>
-
 #include "control/conf.h"
 #include "control/control.h"
 #include "common/collection.h"
 #include "common/debug.h"
 #include "common/metadata.h"
 #include "common/utility.h"
+#include "common/image.h"
+
+#include <stdio.h>
+#include <memory.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <glib.h>
+
 
 #define SELECT_QUERY "select distinct * from %s"
 #define ORDER_BY_QUERY "order by %s"
 #define LIMIT_QUERY "limit ?1, ?2"
 
-#define MAX_QUERY_STRING_LENGTH 4096
 /* Stores the collection query, returns 1 if changed.. */
 static int _dt_collection_store (const dt_collection_t *collection, gchar *query);
 
-typedef struct dt_collection_listener_t
-{
-  void (*callback)(void *);
-  void *data;
-}
-dt_collection_listener_t;
-
 const dt_collection_t *
 dt_collection_new (const dt_collection_t *clone)
 {
@@ -57,7 +51,7 @@ dt_collection_new (const dt_collection_t *clone)
     memcpy (&collection->store,&clone->store,sizeof (dt_collection_params_t));
     collection->where_ext = g_strdup(clone->where_ext);
     collection->query = g_strdup(clone->query);
-    collection->clone =1;
+    collection->clone = 1;
   }
   else  /* else we just initialize using the reset */
     dt_collection_reset (collection);
@@ -85,12 +79,8 @@ int
 dt_collection_update (const dt_collection_t *collection)
 {
   uint32_t result;
-  gchar sq[512]   = {0};   // sort query
-  gchar selq[512] = {0};   // selection query
-  gchar wq[2048]  = {0};   // where query
-  gchar *query=g_malloc (MAX_QUERY_STRING_LENGTH);
-
-  dt_lib_sort_t sort = dt_conf_get_int ("ui_last/combo_sort");
+  gchar *wq, *sq, *selq, *query;
+  wq = sq = selq = query = NULL;
 
   /* build where part */
   if (!(collection->params.query_flags&COLLECTION_QUERY_USE_ONLY_WHERE_EXT))
@@ -100,53 +90,58 @@ dt_collection_update (const dt_collection_t *collection)
     /* add default filters */
     if (collection->params.filter_flags & COLLECTION_FILTER_FILM_ID)
     {
-      g_snprintf (wq,2048,"(film_id = %d)",collection->params.film_id);
+      wq = dt_util_dstrcat(wq, "(film_id = %d)", collection->params.film_id);
       need_operator = 1;
     }
     // DON'T SELECT IMAGES MAKED TO BE DELETED.
-    g_snprintf (wq+strlen(wq),2048-strlen(wq)," %s (flags & %d) != %d", (need_operator)?"and":((need_operator=1)?"":""),DT_IMAGE_REMOVE, DT_IMAGE_REMOVE);
+    wq = dt_util_dstrcat(wq, " %s (flags & %d) != %d", (need_operator)?"and":((need_operator=1)?"":""), DT_IMAGE_REMOVE, DT_IMAGE_REMOVE);
     
 
     if (collection->params.filter_flags & COLLECTION_FILTER_ATLEAST_RATING)
-      g_snprintf (wq+strlen(wq),2048-strlen(wq)," %s (flags & 7) >= %d and (flags & 7) != 6", (need_operator)?"and":((need_operator=1)?"":"") , collection->params.rating);
+      wq = dt_util_dstrcat(wq, " %s (flags & 7) >= %d and (flags & 7) != 6", (need_operator)?"and":((need_operator=1)?"":""), collection->params.rating);
     else if (collection->params.filter_flags & COLLECTION_FILTER_EQUAL_RATING)
-      g_snprintf (wq+strlen(wq),2048-strlen(wq)," %s (flags & 7) == %d", (need_operator)?"and":((need_operator=1)?"":"") , collection->params.rating);
+      wq = dt_util_dstrcat(wq, " %s (flags & 7) == %d", (need_operator)?"and":((need_operator=1)?"":""), collection->params.rating);
 
     if (collection->params.filter_flags & COLLECTION_FILTER_ALTERED)
-      g_snprintf (wq+strlen(wq),2048-strlen(wq)," %s id in (select imgid from history where imgid=id)", (need_operator)?"and":((need_operator=1)?"":"") );
+      wq = dt_util_dstrcat(wq, " %s id in (select imgid from history where imgid=id)", (need_operator)?"and":((need_operator=1)?"":"") );
     else if (collection->params.filter_flags & COLLECTION_FILTER_UNALTERED)
-      g_snprintf (wq+strlen(wq),2048-strlen(wq)," %s id not in (select imgid from history where imgid=id)", (need_operator)?"and":((need_operator=1)?"":"") );
+      wq = dt_util_dstrcat(wq, " %s id not in (select imgid from history where imgid=id)", (need_operator)?"and":((need_operator=1)?"":"") );
 
     /* add where ext if wanted */
     if ((collection->params.query_flags&COLLECTION_QUERY_USE_WHERE_EXT))
-      g_snprintf (wq+strlen(wq),2048-strlen(wq)," %s %s",(need_operator)?"and":"", collection->where_ext);
+      wq = dt_util_dstrcat(wq, " %s %s", (need_operator)?"and":"", collection->where_ext);
   }
   else
-    g_snprintf (wq,512,"%s",collection->where_ext);
-
+    wq = dt_util_dstrcat(wq, "%s", collection->where_ext);
 
+  
 
   /* build select part includes where */
-  if (sort == DT_LIB_SORT_COLOR && (collection->params.query_flags & COLLECTION_QUERY_USE_SORT))
-    g_snprintf (selq,512,"select distinct id from (select * from images where %s) as a left outer join color_labels as b on a.id = b.imgid",wq);
+  if (collection->params.sort == DT_COLLECTION_SORT_COLOR && (collection->params.query_flags & COLLECTION_QUERY_USE_SORT))
+    selq = dt_util_dstrcat(selq, "select distinct id from (select * from images where %s) as a left outer join color_labels as b on a.id = b.imgid", wq);
   else
-    g_snprintf(selq,512, "select distinct id from images where %s",wq);
+    selq = dt_util_dstrcat(selq, "select distinct id from images where %s",wq);
 
 
   /* build sort order part */
   if ((collection->params.query_flags&COLLECTION_QUERY_USE_SORT))
   {
-    if (sort == DT_LIB_SORT_DATETIME)           g_snprintf (sq, 512, ORDER_BY_QUERY, "datetime_taken");
-    else if(sort == DT_LIB_SORT_RATING)         g_snprintf (sq, 512, ORDER_BY_QUERY, "flags & 7 desc");
-    else if(sort == DT_LIB_SORT_FILENAME)       g_snprintf (sq, 512, ORDER_BY_QUERY, "filename");
-    else if(sort == DT_LIB_SORT_ID)             g_snprintf (sq, 512, ORDER_BY_QUERY, "id");
-    else if(sort == DT_LIB_SORT_COLOR)          g_snprintf (sq, 512, ORDER_BY_QUERY, "color desc, filename");
+    sq = dt_collection_get_sort_query(collection);
   }
 
   /* store the new query */
-  g_snprintf (query,MAX_QUERY_STRING_LENGTH,"%s %s%s", selq, sq, (collection->params.query_flags&COLLECTION_QUERY_USE_LIMIT)?" "LIMIT_QUERY:"");
-  result = _dt_collection_store (collection,query);
+  query = dt_util_dstrcat(query, "%s %s%s", selq, sq?sq:"", (collection->params.query_flags&COLLECTION_QUERY_USE_LIMIT)?" "LIMIT_QUERY:"");
+  result = _dt_collection_store(collection, query);
+
+  /* free memory used */
+  if (sq)
+    g_free(sq);
+  g_free(wq);
+  g_free(selq);
   g_free (query);
+
+  dt_collection_hint_message(collection);
+
   return result;
 }
 
@@ -166,6 +161,8 @@ dt_collection_reset(const dt_collection_t *collection)
   params->rating       = dt_conf_get_int("plugins/collection/rating");
   params->query_flags  = dt_conf_get_int("plugins/collection/query_flags");
   params->filter_flags = dt_conf_get_int("plugins/collection/filter_flags");
+  params->sort         = dt_conf_get_int("plugins/collection/sort");
+  params->descending   = dt_conf_get_bool("plugins/collection/descending");
   dt_collection_update_query (collection);
 }
 
@@ -230,16 +227,113 @@ dt_collection_set_rating (const dt_collection_t *collection, uint32_t rating)
   params->rating = rating;
 }
 
+uint32_t
+dt_collection_get_rating (const dt_collection_t *collection)
+{
+  uint32_t i;
+  dt_collection_params_t *params=(dt_collection_params_t *)&collection->params;
+  i = params->rating;
+  i++; /* The enum starts on 0 */
+  return i;
+}
+
+void
+dt_collection_set_sort(const dt_collection_t *collection, dt_collection_sort_t sort, gboolean reverse)
+{
+  dt_collection_params_t *params=(dt_collection_params_t *)&collection->params;
+
+  if(sort != -1)
+    params->sort = sort;
+  if(reverse != -1)
+    params->descending = reverse;
+}
+
+dt_collection_sort_t
+dt_collection_get_sort_field(const dt_collection_t *collection)
+{
+  return collection->params.sort;
+}
+
+gboolean
+dt_collection_get_sort_descending(const dt_collection_t *collection)
+{
+  return collection->params.descending;
+}
+
+gchar *  
+dt_collection_get_sort_query(const dt_collection_t *collection)
+{ 
+  gchar *sq = NULL;
+
+  switch(collection->params.sort)
+  {
+    case DT_COLLECTION_SORT_DATETIME:
+      sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "datetime_taken");
+    break;
+ 
+    case DT_COLLECTION_SORT_RATING:
+      sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "flags & 7 desc");
+    break;
+
+    case DT_COLLECTION_SORT_FILENAME:
+      sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "filename");
+    break;
+
+    case DT_COLLECTION_SORT_ID:
+      sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "id");
+    break;
+
+    case DT_COLLECTION_SORT_COLOR:
+      sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "color desc, filename");
+    break;
+  }
+ 
+ if (collection->params.descending)
+ {
+   switch(collection->params.sort)
+   {
+     case DT_COLLECTION_SORT_DATETIME:
+     case DT_COLLECTION_SORT_FILENAME:
+     case DT_COLLECTION_SORT_ID:
+     {
+       sq = dt_util_dstrcat(sq, " %s", "desc");
+     }
+     break;
+
+     /* These two are special as they are descending in the default view */
+     case DT_COLLECTION_SORT_RATING:
+     {
+       g_free(sq);
+       sq = NULL;
+       sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "flags & 7");
+     }
+     break;
+
+     case DT_COLLECTION_SORT_COLOR:
+     {
+       g_free(sq);
+       sq = NULL;
+       sq = dt_util_dstrcat(sq, ORDER_BY_QUERY, "color, filename");
+     }
+     break;
+   }
+ }
+ return sq;
+}
+
+
 static int
 _dt_collection_store (const dt_collection_t *collection, gchar *query)
 {
-  /* store flags to gconf */
+  /* store flags to conf */
   if (collection == darktable.collection)
   {
     dt_conf_set_int ("plugins/collection/query_flags",collection->params.query_flags);
     dt_conf_set_int ("plugins/collection/filter_flags",collection->params.filter_flags);
     dt_conf_set_int ("plugins/collection/film_id",collection->params.film_id);
     dt_conf_set_int ("plugins/collection/rating",collection->params.rating);
+    dt_conf_set_int ("plugins/collection/sort",collection->params.sort);
+    dt_conf_set_bool ("plugins/collection/descending",collection->params.descending);
   }
 
   /* store query in context */
@@ -256,44 +350,55 @@ uint32_t dt_collection_get_count(const dt_collection_t *collection)
   sqlite3_stmt *stmt = NULL;
   uint32_t count=1;
   const gchar *query = dt_collection_get_query(collection);
-  char countquery[2048]= {0};
-  snprintf(countquery, 2048, "select count(id) %s", query + 18);
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, countquery, -1, &stmt, NULL);
-  DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, 0);
-  DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, -1);
+  gchar *count_query = NULL;
+  count_query = dt_util_dstrcat(count_query, "select count(id) %s", query + 18);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), count_query, -1, &stmt, NULL);
+  if ((collection->params.query_flags&COLLECTION_QUERY_USE_LIMIT)) 
+  {
+    DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, 0);
+    DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, -1);
+  }
+  if(sqlite3_step(stmt) == SQLITE_ROW)
+    count = sqlite3_column_int(stmt, 0);
+  sqlite3_finalize(stmt);
+  g_free(count_query);
+  return count;
+}
+
+uint32_t dt_collection_get_selected_count (const dt_collection_t *collection)
+{
+  sqlite3_stmt *stmt = NULL;
+  uint32_t count=0;
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select count (distinct imgid) from selected_images", -1, &stmt, NULL);
   if(sqlite3_step(stmt) == SQLITE_ROW)
     count = sqlite3_column_int(stmt, 0);
   sqlite3_finalize(stmt);
   return count;
 }
 
-
 GList *dt_collection_get_selected (const dt_collection_t *collection)
 {
   GList *list=NULL;
-  dt_lib_sort_t sort = dt_conf_get_int ("ui_last/combo_sort");
+  gchar *query = NULL;
+  gchar *sq = NULL;
 
   /* get collection order */
-  char sq[512]= {0};
   if ((collection->params.query_flags&COLLECTION_QUERY_USE_SORT))
-  {
-    if (sort == DT_LIB_SORT_DATETIME)           g_snprintf (sq, 512, ORDER_BY_QUERY, "datetime_taken");
-    else if(sort == DT_LIB_SORT_RATING)         g_snprintf (sq, 512, ORDER_BY_QUERY, "flags & 7 desc");
-    else if(sort == DT_LIB_SORT_FILENAME)       g_snprintf (sq, 512, ORDER_BY_QUERY, "filename");
-    else if(sort == DT_LIB_SORT_ID)             g_snprintf (sq, 512, ORDER_BY_QUERY, "id");
-    else if(sort == DT_LIB_SORT_COLOR)          g_snprintf (sq, 512, ORDER_BY_QUERY, "color desc,id");
-  }
+    sq = dt_collection_get_sort_query(collection);
 
 
   sqlite3_stmt *stmt = NULL;
-  char query[2048]= {0};
 
-  if (sort == DT_LIB_SORT_COLOR && (collection->params.query_flags & COLLECTION_QUERY_USE_SORT))
-    g_snprintf (query,512,"select distinct a.imgid as id from (select imgid from selected_images) as a left outer join color_labels as b on a.imgid = b.imgid %s",sq);
-  else
-    g_snprintf(query,512, "select distinct id from images where id in (select imgid from selected_images) %s",sq);
+  /* build the query string */
+  query = dt_util_dstrcat(query, "select distinct id from images ");
 
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db,query, -1, &stmt, NULL);
+  if (collection->params.sort == DT_COLLECTION_SORT_COLOR && (collection->params.query_flags & COLLECTION_QUERY_USE_SORT))
+    query = dt_util_dstrcat(query, "as a left outer join color_labels as b on a.id = b.imgid ");
+
+  query = dt_util_dstrcat(query, "where id in (select imgid from selected_images) %s", sq);
+
+    
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db),query, -1, &stmt, NULL);
 
   while (sqlite3_step (stmt) == SQLITE_ROW)
   {
@@ -301,6 +406,13 @@ GList *dt_collection_get_selected (const dt_collection_t *collection)
     list = g_list_append (list, (gpointer)imgid);
   }
 
+
+  /* free allocated strings */
+  if (sq)
+    g_free(sq);
+
+  g_free(query);
+
   return list;
 }
 
@@ -359,6 +471,18 @@ get_query_string(const int property, const gchar *escaped_text, char *query)
       snprintf(query, 1024, "(id in (select id from meta_data where key = %d and value like '%%%s%%'))",
                DT_METADATA_XMP_DC_RIGHTS, escaped_text);
       break;
+    case 11: // lens
+      snprintf(query, 1024, "(lens like '%%%s%%')", escaped_text);
+      break;
+    case 12: // iso
+      snprintf(query, 1024, "(iso like '%%%s%%')", escaped_text);
+      break;
+    case 13: // aperature
+      snprintf(query, 1024, "(aperture like '%%%s%%')", escaped_text);
+      break;
+    case 14: // filename
+      snprintf(query, 1024, "(filename like '%%%s%%')", escaped_text);
+      break;
 
     default: // case 3: // day
       snprintf(query, 1024, "(datetime_taken like '%%%s%%')", escaped_text);
@@ -366,42 +490,17 @@ get_query_string(const int property, const gchar *escaped_text, char *query)
   }
 }
 
-void
-dt_collection_listener_register(void (*callback)(void *), void *data)
-{
-  dt_collection_listener_t *a = (dt_collection_listener_t *)malloc(sizeof(dt_collection_listener_t));
-  a->callback = callback;
-  a->data = data;
-  darktable.collection_listeners = g_list_append(darktable.collection_listeners, a);
-}
-
-void
-dt_collection_listener_unregister(void (*callback)(void *))
-{
-  GList *i = darktable.collection_listeners;
-  while(i)
-  {
-    dt_collection_listener_t *a = (dt_collection_listener_t *)i->data;
-    GList *ii = g_list_next(i);
-    if(a->callback == callback)
-    {
-      free(a);
-      darktable.collection_listeners = g_list_delete_link(darktable.collection_listeners, i);
-    }
-    i = ii;
-  }
-}
-
 void
 dt_collection_update_query(const dt_collection_t *collection)
 {
   char query[1024], confname[200];
-  char complete_query[4096];
-  int pos = 0;
+  gchar *complete_query = NULL;
 
   const int num_rules = CLAMP(dt_conf_get_int("plugins/lighttable/collect/num_rules"), 1, 10);
   char *conj[] = {"and", "or", "and not"};
-  complete_query[pos++] = '(';
+  
+  complete_query = dt_util_dstrcat(complete_query, "(");
+
   for(int i=0; i<num_rules; i++)
   {
     snprintf(confname, 200, "plugins/lighttable/collect/item%1d", i);
@@ -415,14 +514,16 @@ dt_collection_update_query(const dt_collection_t *collection)
 
     get_query_string(property, escaped_text, query);
 
-    if(i > 0) pos += sprintf(complete_query + pos, " %s %s", conj[mode], query);
-    else pos += sprintf(complete_query + pos, "%s", query);
+    if(i > 0) 
+      complete_query = dt_util_dstrcat(complete_query, " %s %s", conj[mode], query);
+    else 
+      complete_query = dt_util_dstrcat(complete_query, "%s", query);
 
     g_free(escaped_text);
     g_free(text);
   }
-  complete_query[pos++] = ')';
-  complete_query[pos]   = '\0';
+
+  complete_query = dt_util_dstrcat(complete_query, ")");
 
   // printf("complete query: `%s'\n", complete_query);
 
@@ -436,27 +537,42 @@ dt_collection_update_query(const dt_collection_t *collection)
   /* update query and at last the visual */
   dt_collection_update (collection);
 
+  /* free string */
+  g_free(complete_query);
+
   // remove from selected images where not in this query.
   sqlite3_stmt *stmt = NULL;
   const gchar *cquery = dt_collection_get_query(collection);
+  complete_query = NULL;
   if(cquery && cquery[0] != '\0')
   {
-    snprintf(complete_query, 4096, "delete from selected_images where imgid not in (%s)", cquery);
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, complete_query, -1, &stmt, NULL);
+    complete_query = dt_util_dstrcat(complete_query, "delete from selected_images where imgid not in (%s)", cquery);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), complete_query, -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, 0);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, -1);
     sqlite3_step(stmt);
     sqlite3_finalize(stmt);
+
+    /* free allocated strings */
+    g_free(complete_query);
   }
 
-  // notify our listeners:
-  GList *i = darktable.collection_listeners;
-  while(i)
-  {
-    dt_collection_listener_t *a = (dt_collection_listener_t *)i->data;
-    a->callback(a->data);
-    i = g_list_next(i);
-  }
+
+  /* raise signal of collection change, only if this is an orginal */
+  if (!collection->clone)
+    dt_control_signal_raise(darktable.signals, DT_SIGNAL_COLLECTION_CHANGED);
+
+}
+
+void dt_collection_hint_message(const dt_collection_t *collection)
+{
+  /* collection hinting */
+  gchar message[1024];
+  int c = dt_collection_get_count(collection);
+  int cs = dt_collection_get_selected_count(collection);
+  g_snprintf(message, 1024,
+      ngettext("%d image of %d in current collection is selected", "%d images of %d in current collection are selected", cs), cs, c);
+  dt_control_hinter_message(darktable.control, message);
 }
 
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
