@@ -16,12 +16,13 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <lcms2.h>
+#include "common/darktable.h"
 #include "iop/colorout.h"
 #include "control/conf.h"
 #include "control/control.h"
 #include "common/colormatrices.c"
 #include "common/debug.h"
+#include <lcms2.h>
 
 
 /** inverts the given 3x3 matrix */
@@ -613,7 +614,7 @@ dt_colorspaces_create_output_profile(const int imgid)
     const dt_iop_colorout_params_t *params;
     // sqlite:
     sqlite3_stmt *stmt;
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select op_params from history where imgid=?1 and operation='colorout'", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select op_params from history where imgid=?1 and operation='colorout'", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
     if(sqlite3_step(stmt) == SQLITE_ROW)
     {
