@@ -16,9 +16,9 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "common/darktable.h"
 #include "develop/develop.h"
 #include "control/control.h"
-#include "common/darktable.h"
 #include "common/imageio.h"
 #include "common/image_cache.h"
 #include "common/styles.h"
@@ -28,6 +28,7 @@
 
 #include <libxml/encoding.h>
 #include <libxml/xmlwriter.h>
+#include "gui/accelerators.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -56,6 +57,20 @@ typedef struct
   gboolean        in_plugin;
 } StyleData;
 
+static gboolean _apply_style_shortcut_callback(GtkAccelGroup *accel_group,
+                                   GObject *acceleratable,
+                                   guint keyval, GdkModifierType modifier,
+                                   gpointer data)
+{
+    dt_styles_apply_to_selection (data,0);
+    return TRUE;
+}
+
+static void _destroy_style_shortcut_callback(gpointer data,GClosure *closure)
+{
+	g_free(data);
+}
+
 static int32_t dt_styles_get_id_by_name (const char *name);
 
 gboolean
@@ -74,7 +89,7 @@ dt_styles_create_style_header(const char *name, const char *description)
     return FALSE;
   }
   /* first create the style header */
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "insert into styles (name,description) values (?1,?2)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into styles (name,description) values (?1,?2)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, name,strlen (name),SQLITE_STATIC);
   DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 2, description,strlen (description),SQLITE_STATIC);
   sqlite3_step (stmt);
@@ -111,10 +126,10 @@ dt_styles_create_from_image (const char *name,const char *description,int32_t im
       g_strlcat(include,")", 2048);
       char query[4096]= {0};
       sprintf(query,"insert into style_items (styleid,num,module,operation,op_params,enabled,blendop_params) select ?1, num,module,operation,op_params,enabled,blendop_params from history where imgid=?2 and %s",include);
-      DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, query, -1, &stmt, NULL);
+      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), query, -1, &stmt, NULL);
     }
     else
-      DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "insert into style_items (styleid,num,module,operation,op_params,enabled,blendop_params) select ?1, num,module,operation,op_params,enabled,blendop_params from history where imgid=?2", -1, &stmt, NULL);
+      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into style_items (styleid,num,module,operation,op_params,enabled,blendop_params) select ?1, num,module,operation,op_params,enabled,blendop_params from history where imgid=?2", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, imgid);
     sqlite3_step (stmt);
@@ -128,6 +143,15 @@ dt_styles_create_from_image (const char *name,const char *description,int32_t im
 
     dt_styles_save_to_file(name,stylesdir);
 
+      char tmp_accel[1024];
+      gchar* tmp_name = g_strdup(name); // freed by _destro_style_shortcut_callback
+      snprintf(tmp_accel,1024,"styles/Apply %s",name);
+      dt_accel_register_global( tmp_accel, 0, 0);
+      GClosure *closure;
+      closure = g_cclosure_new(
+          G_CALLBACK(_apply_style_shortcut_callback),
+          tmp_name, _destroy_style_shortcut_callback);
+      dt_accel_connect_global(tmp_accel, closure);
     dt_control_log(_("style named '%s' successfully created"),name);
   }
 }
@@ -137,7 +161,7 @@ dt_styles_apply_to_selection(const char *name,gboolean duplicate)
 {
   /* for each selected image apply style */
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select * from selected_images", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select * from selected_images", -1, &stmt, NULL);
   while(sqlite3_step(stmt) == SQLITE_ROW)
   {
     int imgid = sqlite3_column_int (stmt, 0);
@@ -163,14 +187,14 @@ dt_styles_apply_to_image(const char *name,gboolean duplicate, int32_t imgid)
 #if 1
     {
       /* apply on top of history stack */
-      DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select count(num) from history where imgid = ?1", -1, &stmt, NULL);
+      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select count(num) from history where imgid = ?1", -1, &stmt, NULL);
       DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
       if (sqlite3_step (stmt) == SQLITE_ROW) offs = sqlite3_column_int (stmt, 0);
     }
 #else
     {
       /* replace history stack */
-      DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "delete from history where imgid = ?1", -1, &stmt, NULL);
+      DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from history where imgid = ?1", -1, &stmt, NULL);
       DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
       sqlite3_step (stmt);
     }
@@ -178,7 +202,7 @@ dt_styles_apply_to_image(const char *name,gboolean duplicate, int32_t imgid)
     sqlite3_finalize (stmt);
 
     /* copy history items from styles onto image */
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "insert into history (imgid,num,module,operation,op_params,enabled,blendop_params) select ?1, num+?2,module,operation,op_params,enabled,blendop_params from style_items where styleid=?3", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into history (imgid,num,module,operation,op_params,enabled,blendop_params) select ?1, num+?2,module,operation,op_params,enabled,blendop_params from style_items where styleid=?3", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, imgid);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, offs);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 3, id);
@@ -196,13 +220,11 @@ dt_styles_apply_to_image(const char *name,gboolean duplicate, int32_t imgid)
     if (dt_dev_is_current_image(darktable.develop, imgid))
       dt_dev_reload_history_items (darktable.develop);
 
-    /* reimport image */
-    dt_image_t *img = dt_image_cache_get (imgid, 'r');
-    if(img != NULL)
-    {
-      img->force_reimport = 1;
-      dt_image_cache_flush(img);
-    }
+    /* update xmp file */
+    dt_image_synch_xmp(imgid);
+
+    /* remove old obsolete thumbnails */
+    dt_mipmap_cache_remove(darktable.mipmap_cache, imgid);
   }
 }
 
@@ -214,17 +236,20 @@ dt_styles_delete_by_name(const char *name)
   {
     /* delete the style */
     sqlite3_stmt *stmt;
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "delete from styles where rowid = ?1", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from styles where rowid = ?1", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
     sqlite3_step (stmt);
     sqlite3_finalize (stmt);
 
     /* delete style_items belonging to style */
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "delete from style_items where styleid = ?1", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "delete from style_items where styleid = ?1", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
     sqlite3_step (stmt);
     sqlite3_finalize (stmt);
 
+    char tmp_accel[1024];
+    snprintf(tmp_accel,1024,"styles/Apply %s",name);
+    dt_accel_deregister_global(tmp_accel);
   }
 }
 
@@ -236,7 +261,7 @@ dt_styles_get_item_list (const char *name)
   int id=0;
   if ((id=dt_styles_get_id_by_name(name)) != 0)
   {
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select num, operation, enabled from style_items where styleid=?1 order by num desc", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select num, operation, enabled from style_items where styleid=?1 order by num desc", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
     while (sqlite3_step(stmt) == SQLITE_ROW)
     {
@@ -258,7 +283,7 @@ dt_styles_get_list (const char *filter)
   char filterstring[512]= {0};
   sqlite3_stmt *stmt;
   sprintf (filterstring,"%%%s%%",filter);
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select name, description from styles where name like ?1 or description like ?1 order by name", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select name, description from styles where name like ?1 or description like ?1 order by name", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, filterstring, strlen(filterstring), SQLITE_TRANSIENT);
   GList *result = NULL;
   while (sqlite3_step(stmt) == SQLITE_ROW)
@@ -322,7 +347,7 @@ dt_styles_save_to_file(const char *style_name,const char *filedir)
   xmlTextWriterEndElement(writer);
 
   xmlTextWriterStartElement(writer, BAD_CAST "style");
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select num,module,operation,op_params,enabled,blendop_params from style_items where styleid =?1",-1, &stmt,NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select num,module,operation,op_params,enabled,blendop_params from style_items where styleid =?1",-1, &stmt,NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt,1,dt_styles_get_id_by_name(style_name));
   while (sqlite3_step (stmt) == SQLITE_ROW)
   {
@@ -476,7 +501,7 @@ dt_style_plugin_save(StylePluginData *plugin,gpointer styleId)
 {
   int id = GPOINTER_TO_INT(styleId);
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "insert into style_items (styleid,num,module,operation,op_params,enabled,blendop_params) values(?1,?2,?3,?4,?5,?6,?7)", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "insert into style_items (styleid,num,module,operation,op_params,enabled,blendop_params) values(?1,?2,?3,?4,?5,?6,?7)", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, plugin->num);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 3, plugin->module);
@@ -587,7 +612,7 @@ dt_styles_get_description (const char *name)
   gchar *description = NULL;
   if ((id=dt_styles_get_id_by_name(name)) != 0)
   {
-    DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select description from styles where rowid=?1", -1, &stmt, NULL);
+    DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select description from styles where rowid=?1", -1, &stmt, NULL);
     DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, id);
     sqlite3_step(stmt);
     description = (char *)sqlite3_column_text (stmt, 0);
@@ -603,7 +628,7 @@ dt_styles_get_id_by_name (const char *name)
 {
   int id=0;
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, "select rowid from styles where name=?1 order by rowid desc limit 1", -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), "select rowid from styles where name=?1 order by rowid desc limit 1", -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_TEXT(stmt, 1, name,strlen (name),SQLITE_TRANSIENT);
   if(sqlite3_step(stmt) == SQLITE_ROW)
   {
@@ -613,4 +638,47 @@ dt_styles_get_id_by_name (const char *name)
   return id;
 }
 
+void init_styles_key_accels()
+{
+  GList *result = dt_styles_get_list("");
+  if (result)
+  {
+    do
+    {
+      dt_style_t *style = (dt_style_t *)result->data;
+      char tmp_accel[1024];
+      snprintf(tmp_accel,1024,"styles/Apply %s",style->name);
+      dt_accel_register_global( tmp_accel, 0, 0);
+
+      g_free(style->name);
+      g_free(style->description);
+      g_free(style);
+    }
+    while ((result=g_list_next(result))!=NULL);
+  }
+}
+
+void connect_styles_key_accels()
+{
+  GList *result = dt_styles_get_list("");
+  if (result)
+  {
+    do
+    {
+      GClosure *closure;
+      dt_style_t *style = (dt_style_t *)result->data;
+      closure = g_cclosure_new(
+          G_CALLBACK(_apply_style_shortcut_callback),
+          style->name, _destroy_style_shortcut_callback);
+      char tmp_accel[1024];
+      snprintf(tmp_accel,1024,"styles/Apply %s",style->name);
+      dt_accel_connect_global(tmp_accel, closure);
+
+      //g_free(style->name); freed at closure destruction
+      g_free(style->description);
+      g_free(style);
+    }
+    while ((result=g_list_next(result))!=NULL);
+  }
+}
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
