@@ -55,13 +55,16 @@ void dt_tag_detach_by_string(const char *name, gint imgid);
 /** retreives a list of tags of specified imgid \param[out] result a list of dt_tag_t. */
 uint32_t dt_tag_get_attached(gint imgid,GList **result);
 
-/** retreives a list of suggested tags matching keyword. \param[in] keyword the keyword to search \param[out] result a pointer to list populated with result. \return the count \note the limit of result is decided by gconf value "xxx" */
+/** retreives a list of suggested tags matching keyword. \param[in] keyword the keyword to search \param[out] result a pointer to list populated with result. \return the count \note the limit of result is decided by conf value "xxx" */
 uint32_t dt_tag_get_suggestions(const gchar *keyword, GList **result);
 
-/** retreives a list of recent tags used. \param[out] result a pointer to list populated with result. \return the count \note the limit of result is decided by gconf value "xxx" */
+/** retreives a list of recent tags used. \param[out] result a pointer to list populated with result. \return the count \note the limit of result is decided by conf value "xxx" */
 uint32_t dt_tag_get_recent_used(GList **result);
 
 /** frees the memory of a result set. */
 void dt_tag_free_result(GList **result);
 
+/** reorgnize tags */
+void dt_tag_reorganize(const gchar *source, const gchar *dest);
+
 #endif
