@@ -15,38 +15,37 @@
     You should have received a copy of the GNU General Public License
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
-
-
-
-#include <stdio.h>
-#include <glib.h>
-
-#include "common/camera_control.h"
 #include "common/darktable.h"
+#include "common/camera_control.h"
 #include "common/utility.h"
 #include "views/view.h"
 #include "control/conf.h"
 #include "control/jobs/camera_jobs.h"
 #include "gui/gtk.h"
 
+#include <stdio.h>
+#include <glib.h>
+
+
 int32_t dt_captured_image_import_job_run(dt_job_t *job)
 {
   dt_captured_image_import_t *t = (dt_captured_image_import_t *)job->param;
 
   char message[512]= {0};
   snprintf(message, 512, _("importing image %s"), t->filename);
-  const dt_gui_job_t *j = dt_gui_background_jobs_new( DT_JOB_SINGLE, message );
+  const guint *jid = dt_control_backgroundjobs_create(darktable.control, 0, message );
 
   int id = dt_image_import(t->film_id, t->filename, TRUE);
   if(id)
   {
     //dt_film_open(1);
-    dt_view_film_strip_set_active_image(darktable.view_manager,id);
-    dt_control_queue_draw_all();
+    dt_view_filmstrip_set_active_image(darktable.view_manager,id);
+    dt_control_queue_redraw();
     //dt_ctl_switch_mode_to(DT_DEVELOP);
   }
-  dt_gui_background_jobs_set_progress( j , 1.0 );
-  dt_gui_background_jobs_destroy (j);
+ 
+  dt_control_backgroundjobs_progress(darktable.control, jid, 1.0);
+  dt_control_backgroundjobs_destroy(darktable.control, jid);
   return 0;
 }
 
@@ -62,12 +61,11 @@ void dt_captured_image_import_job_init(dt_job_t *job,uint32_t filmid, const char
 int32_t dt_camera_capture_job_run(dt_job_t *job)
 {
   dt_camera_capture_t *t=(dt_camera_capture_t*)job->param;
-  int total = t->count * t->brackets;
+  int total = t->brackets ? t->count * t->brackets : t->count;
   char message[512]= {0};
-  snprintf(message, 512, ngettext ("capturing %d image", "capturing %d images", total), total );
   double fraction=0;
-  const dt_gui_job_t *j = dt_gui_background_jobs_new( DT_JOB_PROGRESS, message );
-
+  snprintf(message, 512, ngettext ("capturing %d image", "capturing %d images", total), total );
+  
   /* try to get exp program mode for nikon */
   char *expprogram = (char *)dt_camctl_camera_get_property(darktable.camctl, NULL, "expprogram");
   
@@ -100,12 +98,13 @@ int32_t dt_camera_capture_job_run(dt_job_t *job)
     if(t->brackets)
     {
       dt_control_log(_("please set your camera to manual mode first!"));
-      dt_gui_background_jobs_set_progress(j, 1.001f);
-      dt_gui_background_jobs_destroy(j);
       return 1;
     }
   }
 
+  /* create the bgjob plate */
+  const guint *jid  = dt_control_backgroundjobs_create(darktable.control, 0, message);
+
   GList *current_value = g_list_find(values,orginal_value);
   for(int i=0; i<t->count; i++)
   {
@@ -136,8 +135,9 @@ int32_t dt_camera_capture_job_run(dt_job_t *job)
 
       // Capture image
       dt_camctl_camera_capture(darktable.camctl,NULL);
-      fraction+=1.0/total;
-      dt_gui_background_jobs_set_progress( j, fraction);
+
+      fraction += 1.0/total;
+      dt_control_backgroundjobs_progress(darktable.control, jid, fraction);
     }
 
     // lets reset to orginal value before continue
@@ -152,7 +152,9 @@ int32_t dt_camera_capture_job_run(dt_job_t *job)
       g_usleep(t->delay*G_USEC_PER_SEC);
 
   }
-  dt_gui_background_jobs_destroy (j);
+
+  dt_control_backgroundjobs_destroy(darktable.control, jid);
+
 
   // free values
   if(values)
@@ -290,11 +292,12 @@ void _camera_image_downloaded(const dt_camera_t *camera,const char *filename,voi
   dt_control_log(_("%d/%d imported to %s"), t->import_count+1,g_list_length(t->images), g_path_get_basename(filename));
 
   t->fraction+=1.0/g_list_length(t->images);
-  dt_gui_background_jobs_set_progress( t->bgj, t->fraction );
+  
+  dt_control_backgroundjobs_progress(darktable.control, t->bgj, t->fraction );
 
   if( dt_conf_get_bool("plugins/capture/camera/import/backup/enable") == TRUE )
   {
-    // Backup is enabled, let's initialize a backup job of imported image...
+    // Backup is enable, let's initialize a backup job of imported image...
     char *base=dt_conf_get_string("plugins/capture/storage/basedirectory");
     char *fixed_base=dt_util_fix_path(base);
     dt_variables_expand( t->vp, fixed_base, FALSE );
@@ -382,7 +385,7 @@ int32_t dt_camera_import_job_run(dt_job_t *job)
     int total = g_list_length( t->images );
     char message[512]= {0};
     sprintf(message, ngettext ("importing %d image from camera", "importing %d images from camera", total), total );
-    t->bgj = dt_gui_background_jobs_new( DT_JOB_PROGRESS, message);
+    t->bgj = dt_control_backgroundjobs_create(darktable.control, 0, message);
 
     // Switch to new filmroll
     dt_film_open(t->film->id);
@@ -399,7 +402,7 @@ int32_t dt_camera_import_job_run(dt_job_t *job)
     dt_camctl_register_listener(darktable.camctl,&listener);
     dt_camctl_import(darktable.camctl,t->camera,t->images,dt_conf_get_bool("plugins/capture/camera/import/delete_originals"));
     dt_camctl_unregister_listener(darktable.camctl,&listener);
-    dt_gui_background_jobs_destroy (t->bgj);
+    dt_control_backgroundjobs_destroy(darktable.control, t->bgj);
     dt_variables_params_destroy(t->vp);
   }
   else
