@@ -16,12 +16,6 @@
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#define _XOPEN_SOURCE 500
-#include <stdlib.h>
-double drand48(void);
-void srand48(long int);
-#include <unistd.h>
-#include <inttypes.h>
 #include "common/darktable.h"
 #include "common/debug.h"
 #include "common/collection.h"
@@ -31,15 +25,24 @@ void srand48(long int);
 #include "common/imageio.h"
 #include "common/imageio_module.h"
 #include "control/conf.h"
+
 #include <GL/gl.h>
 #include <GL/glu.h>
 #include <SDL/SDL.h>
+#include <stdlib.h>
+double drand48(void);
+void srand48(long int);
+#include <sys/time.h>
+#include <unistd.h>
+int usleep(useconds_t usec);
+#include <inttypes.h>
 
 int running;
 int width, height;
 uint32_t random_state;
 int use_random;
 float *pixels;
+uint32_t scramble = 0;
 
 int init(int argc, char *arg[])
 {
@@ -47,6 +50,10 @@ int init(int argc, char *arg[])
   int bpp = 0;
   int flags = 0;
 
+  struct timeval time;
+  gettimeofday(&time, NULL);
+  scramble = time.tv_sec + time.tv_usec;
+
   if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
   {
     fprintf( stderr, "[%s] video initialization failed: %s\n", arg[0], SDL_GetError() );
@@ -55,6 +62,11 @@ int init(int argc, char *arg[])
 
   info = SDL_GetVideoInfo( );
 
+  if (info == NULL) {
+    fprintf( stderr, "[%s] video info failed: %s\n", arg[0], SDL_GetError());
+    exit(1);
+  }
+
   width  = info->current_w;
   height = info->current_h;
   pixels = (float *)malloc(sizeof(float)*4*width*height);
@@ -108,6 +120,10 @@ int init(int argc, char *arg[])
   gluOrtho2D(0, 1, 1, 0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
+
+  glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
+  glClear(GL_COLOR_BUFFER_BIT);
+  return 1;
 }
 
 static void
@@ -143,10 +159,15 @@ pump_events()
 static void
 update(const int frame)
 {
+  // copy over the buffer, so we can blend smoothly.
+  glReadBuffer (GL_FRONT);
+  glDrawBuffer (GL_BACK);
+  glCopyPixels (0, 0, width, height, GL_COLOR);
   glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, width, height, GL_RGBA, GL_FLOAT, pixels);
-  if(frame < 10)
+  if(frame < 18)
   {
-    glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
+    // glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
   }
   else
@@ -163,6 +184,7 @@ update(const int frame)
   glTexCoord2d(1.0f, 1.0f);
   glVertex2f(1.0f, 1.0f);
   glEnd();
+  // display the back buffer
   SDL_GL_SwapBuffers();
 }
 
@@ -180,12 +202,14 @@ write_image (dt_imageio_module_data_t *data, const char *filename, const void *i
   float *out = pixels + (offy * width  + offx )* 4;
   const float *rd = in;
   memset(pixels, 0, 4*sizeof(float)*width*height);
+  const float alpha = 0.2f;
+  for(int i=3;i<4*width*height;i+=4) pixels[i] = 0.2f;
   for(int j=0; j<MIN(data->height, height); j++)
   {
     for(int i=0; i<MIN(data->width, width); i++)
     {
       for(int c=0; c<3; c++) out[4*i+c] = rd[4*i+c];
-      out[4*i+3] = .5f;
+      out[4*i+3] = alpha;
     }
     out += 4*width;
     rd  += 4*data->width;
@@ -201,7 +225,7 @@ uint32_t next_random()
   i = ((i & 0x0f0f0f0f) <<  4) | ((i & 0xf0f0f0f0) >> 4);
   i = ((i & 0x33333333) <<  2) | ((i & 0xcccccccc) >> 2);
   i = ((i & 0x55555555) <<  1) | ((i & 0xaaaaaaaa) >> 1);
-  return i;
+  return i ^ scramble;
 }
 
 static int
@@ -233,7 +257,7 @@ process_next_image()
   const gchar *query = dt_collection_get_query (darktable.collection);
   if(!query) return 1;
   sqlite3_stmt *stmt;
-  DT_DEBUG_SQLITE3_PREPARE_V2(darktable.db, query, -1, &stmt, NULL);
+  DT_DEBUG_SQLITE3_PREPARE_V2(dt_database_get(darktable.db), query, -1, &stmt, NULL);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 1, rand);
   DT_DEBUG_SQLITE3_BIND_INT(stmt, 2, rand+1);
   if(sqlite3_step(stmt) == SQLITE_ROW)
@@ -243,9 +267,7 @@ process_next_image()
   if(id)
   {
     // get image from cache
-    dt_image_t *img = dt_image_cache_get(id, 'r');
-    dt_imageio_export(img, "unused", &buf, &dat);
-    dt_image_cache_release(img, 'r');
+    dt_imageio_export(id, "unused", &buf, &dat);
   }
   return 0;
 }
@@ -266,7 +288,7 @@ int main(int argc, char *arg[])
   // init dt without gui:
   if(dt_init(argc, arg, 0)) exit(1);
   // use system color profile, if we can:
-  const gchar *oldprofile = dt_conf_get_string("plugins/lighttable/export/iccprofile");
+  gchar *oldprofile = dt_conf_get_string("plugins/lighttable/export/iccprofile");
   const gchar *overprofile = "X profile";
   dt_conf_set_string("plugins/lighttable/export/iccprofile", overprofile);
   running = init(argc, arg);
@@ -277,12 +299,17 @@ int main(int argc, char *arg[])
     pump_events();
     if(!running) break;
     if(process_next_image()) break;
-    for(int k=0; k<=10; k++)
+    for(int k=0; k<=18; k++)
     {
       update(k);
-      usleep(30000);
+      usleep(10000);
+    }
+    for(int k=0;k<100;k++)
+    {
+      pump_events();
+      if(!running) break;
+      usleep(35000);
     }
-    usleep(3000000);
   }
   if(oldprofile) {
     dt_conf_set_string("plugins/lighttable/export/iccprofile", oldprofile);
