@@ -82,7 +82,6 @@ gui_reset (dt_imageio_module_storage_t *self)
 int
 store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_format_t *format, dt_imageio_module_data_t *fdata, const int num, const int total)
 {
-  dt_image_t *img = dt_image_cache_get(imgid, 'r');
   dt_imageio_email_t *d = (dt_imageio_email_t *)sdata;
 
   _email_attachment_t *attachment = ( _email_attachment_t *)malloc(sizeof(_email_attachment_t));
@@ -95,15 +94,14 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
   g_mkdir_with_parents(tmpdir,0700);
 
   char dirname[4096];
-  dt_image_full_path(img->id, dirname, 1024);
+  dt_image_full_path(imgid, dirname, 1024);
   const gchar * filename = g_path_get_basename( dirname );
   gchar * end = g_strrstr( filename,".")+1;
   g_strlcpy( end, format->extension(fdata), sizeof(dirname)-(end-dirname));
 
   attachment->file = g_build_filename( tmpdir, filename, (char *)NULL );
 
-  dt_imageio_export(img, attachment->file, format, fdata);
-  dt_image_cache_release(img, 'r');
+  dt_imageio_export(imgid, attachment->file, format, fdata);
 
   char *trunc = attachment->file + strlen(attachment->file) - 32;
   if(trunc < attachment->file) trunc = attachment->file;
@@ -148,58 +146,22 @@ finalize_store(dt_imageio_module_storage_t *self, void *params)
   gchar uri[4096]= {0};
   gchar body[4096]= {0};
   gchar attachments[4096]= {0};
-  gchar *defaultHandler=NULL;
   gchar *uriFormat=NULL;
   gchar *subject="images exported from darktable";
-  gchar *imageBodyFormat="%s %s\n"; // filename, exif oneliner
+  gchar *imageBodyFormat="%s %s"; // filename, exif oneliner
   gchar *attachmentFormat=NULL;
   gchar *attachmentSeparator="";
 
-  if(  strlen( defaultHandler = dt_conf_get_string("plugins/imageio/storage/email/client") ) <= 0 )
-  {
-#ifdef HAVE_GCONF
-    defaultHandler = gconf_client_get_string (darktable.conf->gconf, "/desktop/gnome/url-handlers/mailto/command", NULL);
-#endif
-  }
-
-  if( defaultHandler )
-  {
-    // Detected a default handler let's do special case handling for each email client..
-    if( g_strrstr(defaultHandler,"thunderbird") )
-    {
-      uriFormat="thunderbird -compose \"to='',subject='%s',body='%s',attachment='%s'\"";   // subject, body, and list of attachments with format "<filename>,"
-      attachmentFormat="%s";
-      attachmentSeparator=",";
-      goto proceed;
-    }
-    else if( g_strrstr(defaultHandler,"kmail") )
-    {
-      // When I enter the mailto:... in konqueror everything is ok, yet from dt we have no attachements. WTF?
-      // So we launch it directly.
-      uriFormat="kmail --composer --subject \"%s\" --body \"%s\" --attach \"%s\"";   // subject, body, and list of attachments with format "--attach <filename> "
-      attachmentFormat="%s";
-      attachmentSeparator="\" --attach \"";
-      goto proceed;
-    }
-    else if( g_strrstr(defaultHandler,"evolution") )
-    {
-      uriFormat="evolution \"mailto:?subject=%s&body=%s%s\"";   // subject, body, and list of attachments with format "--attach <filename> "
-      attachmentFormat="&attachment=file://%s";
-      goto proceed;
-    }
-  }
-
   // If no default handler detected above, we use gtk_show_uri with mailto:// and hopes things goes right..
-  uriFormat="mailto:?subject=%s&body=%s%s";   // subject, body, and list of attachments with format &attachment=<filename>
-  attachmentFormat="&attachment=file://%s";
+  uriFormat="xdg-email --subject \"%s\" --body \"%s\" %s &";   // subject, body, and list of attachments with format:
+  attachmentFormat=" --attach \"%s\"";
 
-proceed: ; // Let's build up uri / command
   while( d->images )
   {
     gchar exif[256]= {0};
     _email_attachment_t *attachment=( _email_attachment_t *)d->images->data;
     const gchar *filename = g_path_get_basename( attachment->file );
-    dt_image_t *img = dt_image_cache_get( attachment->imgid, 'r');
+    const dt_image_t *img = dt_image_cache_read_get(darktable.image_cache, attachment->imgid);
     dt_image_print_exif( img, exif, 256 );
     g_snprintf(body+strlen(body),4096-strlen(body), imageBodyFormat, filename, exif );
 
@@ -208,7 +170,7 @@ proceed: ; // Let's build up uri / command
 
     g_snprintf(attachments+strlen(attachments),4096-strlen(attachments), attachmentFormat, attachment->file );
     // Free attachment item and remove
-    dt_image_cache_release(img, 'r');
+    dt_image_cache_read_release(darktable.image_cache, img);
     g_free( d->images->data );
     d->images = g_list_remove( d->images, d->images->data );
   }
@@ -216,11 +178,13 @@ proceed: ; // Let's build up uri / command
   // build uri and launch before we quit...
   g_snprintf( uri, 4096,  uriFormat, subject, body, attachments );
 
-  // So what should we do...
-  if( strncmp( uri, "mailto:", 7) == 0 )
-    gtk_show_uri(NULL,uri,GDK_CURRENT_TIME,NULL);
-  else // Launch subprocess
-    if(system( uri ) < 0) fprintf(stderr, "[email] could not launch subprocess!\n");
+  fprintf(stderr, "[email] launching `%s'\n", uri);
+  if(system( uri ) < 0)
+  {
+    // TODO: after string freeze is broken again, report to ui:
+    // dt_control_log(_("could not launch email client!"));
+    fprintf(stderr, "[email] could not launch subprocess!\n");
+  }
 }
 
 int supported(struct dt_imageio_module_storage_t *storage, struct dt_imageio_module_format_t *format)
