@@ -18,12 +18,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdlib.h>
-#include <math.h>
-#include <string.h>
-#include <gtk/gtk.h>
-#include <inttypes.h>
-#include <gdk/gdkkeysyms.h>
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "control/control.h"
@@ -39,6 +33,13 @@
 #include "gui/draw.h"
 #include "gui/presets.h"
 
+#include <stdlib.h>
+#include <math.h>
+#include <string.h>
+#include <gtk/gtk.h>
+#include <inttypes.h>
+#include <gdk/gdkkeysyms.h>
+
 DT_MODULE(1)
 
 typedef struct dt_iop_borders_params_t
@@ -106,6 +107,7 @@ modify_roi_out(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piec
   dt_iop_borders_data_t *d = (dt_iop_borders_data_t *)piece->data;
 
   const float size = fabsf(d->size);
+  if(size == 0) return;
   if(d->size < 0.0f)
   {
     // this means: relative to width and constant for height as well:
@@ -127,8 +129,8 @@ modify_roi_out(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piec
   }
 
   // sanity check.
-  if(roi_out->width  < 1) roi_out->width  = 1;
-  if(roi_out->height < 1) roi_out->height = 1;
+  roi_out->width = CLAMP(roi_out->width, 1, 2*roi_in->width);
+  roi_out->height = CLAMP(roi_out->height, 1, 2*roi_in->height);
 }
 
 // 2nd pass: which roi would this operation need as input to fill the given output region?
@@ -166,7 +168,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
   const int by = MAX(bh/2 - roi_out->y, 0);
 
   // sse-friendly color copy (stupidly copy whole buffer, /me lazy ass)
-  const float col[4] = {d->color[0], d->color[1], d->color[2], 0.0f};
+  const float col[4] = {d->color[0], d->color[1], d->color[2], 1.0f};
   float *buf = (float *)ovoid;
   for(int k=0;k<roi_out->width*roi_out->height;k++, buf+=4) memcpy(buf, col, sizeof(float)*4);
   // blit image inside border and fill border with bg color
@@ -201,11 +203,11 @@ void init_presets (dt_iop_module_so_t *self)
 {
   dt_iop_borders_params_t p = (dt_iop_borders_params_t)
   {
-    {0.0f, 0.0f, 0.0f}, 3.0f/2.0f, 0.1f
+    {1.0f, 1.0f, 1.0f}, 3.0f/2.0f, 0.1f
   };
-  dt_gui_presets_add_generic(_("15:10 postcard black"), self->op, self->version(), &p, sizeof(p), 1);
-  p.color[0] = p.color[1] = p.color[2] = 1.0f;
   dt_gui_presets_add_generic(_("15:10 postcard white"), self->op, self->version(), &p, sizeof(p), 1);
+  p.color[0] = p.color[1] = p.color[2] = 0.0f;
+  dt_gui_presets_add_generic(_("15:10 postcard black"), self->op, self->version(), &p, sizeof(p), 1);
 }
 
 static void
@@ -213,6 +215,11 @@ request_pick_toggled(GtkToggleButton *togglebutton, dt_iop_module_t *self)
 {
   self->request_color_pick = gtk_toggle_button_get_active(togglebutton);
   if(darktable.gui->reset) return;
+  
+  /* use point sample */
+  if (self->request_color_pick)
+    dt_lib_colorpicker_set_point(darktable.lib, 0.5, 0.5);
+  
   if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
   dt_iop_request_focus(self);
 }
@@ -255,7 +262,7 @@ aspect_changed (GtkComboBox *combo, dt_iop_module_t *self)
   int which = gtk_combo_box_get_active(combo);
   if (which < 0)
   {
-    p->aspect = self->dev->image->width/(float)self->dev->image->height;
+    p->aspect = self->dev->image_storage.width/(float)self->dev->image_storage.height;
     gchar *text = gtk_combo_box_get_active_text(combo);
     if(text)
     {
@@ -272,7 +279,7 @@ aspect_changed (GtkComboBox *combo, dt_iop_module_t *self)
   }
   else if (which < 9)
   {
-    if(self->dev->image->height > self->dev->image->width)
+    if(self->dev->image_storage.height > self->dev->image_storage.width)
       p->aspect = 1.0/g->aspect_ratios[which];
     else
       p->aspect = g->aspect_ratios[which];
@@ -303,7 +310,7 @@ colorpick_callback (GtkDarktableButton *button, dt_iop_module_t *self)
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
 
   GtkColorSelectionDialog  *csd = GTK_COLOR_SELECTION_DIALOG(gtk_color_selection_dialog_new(_("select frame color")));
-  gtk_window_set_transient_for(GTK_WINDOW(csd), GTK_WINDOW(darktable.gui->widgets.main_window));
+  gtk_window_set_transient_for(GTK_WINDOW(csd), GTK_WINDOW(dt_ui_main_window(darktable.gui->ui)));
   g_signal_connect (G_OBJECT (csd->ok_button), "clicked",
                     G_CALLBACK (colorpick_button_callback), csd);
   g_signal_connect (G_OBJECT (csd->cancel_button), "clicked",
@@ -364,7 +371,7 @@ void init(dt_iop_module_t *module)
   module->default_enabled = 0;
   module->params_size = sizeof(dt_iop_borders_params_t);
   module->gui_data = NULL;
-  module->priority = 934; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 960; // module order created by iop_dependencies.py, do not edit!
 }
 
 void cleanup(dt_iop_module_t *module)
@@ -407,17 +414,12 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect), _("DIN"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect), _("16:9"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect), _("constant border"));
+  dt_gui_key_accel_block_on_focus(gtk_bin_get_child(GTK_BIN(g->aspect)));
 
   g_signal_connect (G_OBJECT (g->aspect), "changed", G_CALLBACK (aspect_changed), self);
   g_object_set(G_OBJECT(g->aspect), "tooltip-text", _("set the aspect ratio (w:h)\npress ctrl-x to swap sides"), (char *)NULL);
 
   gtk_table_attach(GTK_TABLE(self->widget), GTK_WIDGET(g->aspect), 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
-  GtkWidget *button = dtgtk_button_new(dtgtk_cairo_paint_aspectflip, CPF_STYLE_FLAT);
-  g->swap_button = button;
-  // TODO: what about this?
-  //g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (aspect_flip), self);
-  g_object_set(G_OBJECT(button), "tooltip-text", _("swap the aspect ratio"), (char *)NULL);
-  gtk_table_attach(GTK_TABLE(self->widget), button, 2, 3, 1, 2, GTK_EXPAND, 0, 0, 0);
 
   g->colorpick = DTGTK_BUTTON(dtgtk_button_new(dtgtk_cairo_paint_color, CPF_IGNORE_FG_STATE));
   gtk_widget_set_size_request(GTK_WIDGET(g->colorpick), 24, 24);
@@ -431,10 +433,10 @@ void gui_init(struct dt_iop_module_t *self)
   g_signal_connect(G_OBJECT(tb), "toggled", G_CALLBACK(request_pick_toggled), self);
   gtk_table_attach(GTK_TABLE(self->widget), tb, 2, 3, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
-  g->aspect_ratios[0] = self->dev->image->width/(float)self->dev->image->height;
+  g->aspect_ratios[0] = self->dev->image_storage.width/(float)self->dev->image_storage.height;
   if(g->aspect_ratios[0] < 1.0f)
     g->aspect_ratios[0] = 1.0f / g->aspect_ratios[0];
-  g->aspect_ratios[1] = 1.6280f;
+  g->aspect_ratios[1] = PHI;
   g->aspect_ratios[2] = 2.0f/1.0f;
   g->aspect_ratios[3] = 3.0f/2.0f;
   g->aspect_ratios[4] = 4.0f/3.0f;
@@ -450,16 +452,16 @@ void reload_defaults(dt_iop_module_t *self)
 {
   dt_iop_borders_params_t tmp = (dt_iop_borders_params_t)
   {
-    {0.0f, 0.0f, 0.0f}, 3.0f/2.0f, 0.1f
+    {1.0f, 1.0f, 1.0f}, 3.0f/2.0f, 0.1f
   };
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
   if(self->dev->gui_attached && g)
   {
-    g->aspect_ratios[1] = self->dev->image->width/(float)self->dev->image->height;
+    g->aspect_ratios[1] = self->dev->image_storage.width/(float)self->dev->image_storage.height;
     if(g->aspect_ratios[1] < 1.0f)
       g->aspect_ratios[1] = 1.0f / g->aspect_ratios[1];
   }
-  if(self->dev->image->height > self->dev->image->width)
+  if(self->dev->image_storage.height > self->dev->image_storage.width)
     tmp.aspect = 1.0f/tmp.aspect;
   memcpy(self->params, &tmp, sizeof(dt_iop_borders_params_t));
   memcpy(self->default_params, &tmp, sizeof(dt_iop_borders_params_t));
