@@ -1,6 +1,7 @@
 /*
     This file is part of darktable,
     copyright (c) 2009--2011 johannes hanika.
+    copyright (c) 2012 henrik andersson.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,14 +19,6 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdlib.h>
-#include <math.h>
-#include <string.h>
-#include <gtk/gtk.h>
-#include <inttypes.h>
-#include <gdk/gdkkeysyms.h>
-#include <assert.h>
-
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "control/control.h"
@@ -41,10 +34,18 @@
 #include "gui/draw.h"
 #include "gui/presets.h"
 
+#include <stdlib.h>
+#include <math.h>
+#include <string.h>
+#include <gtk/gtk.h>
+#include <inttypes.h>
+#include <gdk/gdkkeysyms.h>
+#include <assert.h>
+
 DT_MODULE(3)
 
 // number of gui ratios in combo box
-#define NUM_RATIOS 9
+#define NUM_RATIOS 10
 
 /** flip H/V, rotate an image, then clip the buffer. */
 typedef enum dt_iop_clipping_flags_t
@@ -60,6 +61,8 @@ typedef struct dt_iop_clipping_params_t
 }
 dt_iop_clipping_params_t;
 
+/* calculate the aspect ratios for current image */
+static void _iop_clipping_update_ratios(dt_iop_module_t *self);
 
 int
 legacy_params (dt_iop_module_t *self, const void *const old_params, const int old_version, void *new_params, const int new_version)
@@ -100,10 +103,16 @@ typedef struct dt_iop_clipping_gui_data_t
   GtkDarktableToggleButton *flipHorGoldenGuide, *flipVerGoldenGuide;
   GtkCheckButton *goldenSectionBox, *goldenSpiralSectionBox, *goldenSpiralBox, *goldenTriangleBox;
 
+  float button_down_x, button_down_y;
   float button_down_zoom_x, button_down_zoom_y, button_down_angle; // position in image where the button has been pressed.
+  /* current clip box */
   float clip_x, clip_y, clip_w, clip_h, handle_x, handle_y;
+  /* last committed clip box */
   float old_clip_x, old_clip_y, old_clip_w, old_clip_h;
-  int cropping, straightening, applied;
+  /* last box before change */
+  float prev_clip_x, prev_clip_y, prev_clip_w, prev_clip_h;
+
+  int cropping, straightening, applied, center_lock;
   float aspect_ratios[NUM_RATIOS];
   float current_aspect;
 }
@@ -156,7 +165,7 @@ const char *name()
 int
 groups ()
 {
-  return IOP_GROUP_CORRECT;
+  return IOP_GROUP_BASIC;
 }
 
 int
@@ -467,7 +476,7 @@ void gui_focus (struct dt_iop_module_t *self, gboolean in)
       g->clip_h = p->ch - p->cy;
       // flip one bit to trigger the cache:
       uint32_t hack = *(uint32_t*)&p->cy;
-      hack ++;
+      hack ^= 1;
       p->cy = *(float *)&hack;
       if(!darktable.gui->reset)
         dt_dev_add_history_item(darktable.develop, self, TRUE);
@@ -583,34 +592,8 @@ apply_box_aspect(dt_iop_module_t *self, int grab)
   }
 }
 
-void init_presets (dt_iop_module_so_t *self)
-{
-  dt_iop_clipping_params_t p = (dt_iop_clipping_params_t)
-  {
-    0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0
-  };
-  DT_DEBUG_SQLITE3_EXEC(darktable.db, "begin", NULL, NULL, NULL);
-  p.angle = 90.0f;
-  dt_gui_presets_add_generic(_("rotate by  90"), self->op, self->version(), &p, sizeof(p), 1);
-  p.angle = -90.0f;
-  dt_gui_presets_add_generic(_("rotate by -90"), self->op, self->version(), &p, sizeof(p), 1);
-  p.angle = 180.0f;
-  dt_gui_presets_add_generic(_("rotate by 180"), self->op, self->version(), &p, sizeof(p), 1);
-  DT_DEBUG_SQLITE3_EXEC(darktable.db, "commit", NULL, NULL, NULL);
-}
-
 void reload_defaults(dt_iop_module_t *self)
 {
-  dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
-  if(self->dev->gui_attached && g)
-  {
-    g->aspect_ratios[1] = self->dev->image->width/(float)self->dev->image->height;
-    if(g->aspect_ratios[1] < 1.0f)
-      g->aspect_ratios[1] = 1.0f / g->aspect_ratios[1];
-    
-    if(g->current_aspect > 1.0f && self->dev->image->height > self->dev->image->width)
-      g->current_aspect = 1.0f/g->current_aspect;
-  }
   dt_iop_clipping_params_t tmp = (dt_iop_clipping_params_t)
   {
     0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 0.0f
@@ -633,8 +616,9 @@ aspect_free_activated (GtkEntry *entry, dt_iop_module_t *self)
       *c = '\0';
       c++;
       g->current_aspect = atof(text) / atof(c);
+      dt_conf_set_int("plugins/darkroom/clipping/custom_aspect", g->current_aspect);
       apply_box_aspect(self, 5);
-      dt_control_queue_draw_all();
+      dt_control_queue_redraw_center();
       dt_iop_request_focus(self);
     }
     g_free(text);
@@ -657,7 +641,7 @@ aspect_presets_changed (GtkComboBox *combo, dt_iop_module_t *self)
       snprintf(text, 128, "%.3f:1", g->current_aspect);
       gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo))), text);
       apply_box_aspect(self, 5);
-      dt_control_queue_draw_all();
+      dt_control_queue_redraw_center();
     }
     // user is typing, don't overwrite it.
     g->current_aspect = -2.0f;
@@ -667,12 +651,11 @@ aspect_presets_changed (GtkComboBox *combo, dt_iop_module_t *self)
   else if (which < NUM_RATIOS)
   {
     dt_conf_set_int("plugins/darkroom/clipping/aspect_preset", which);
-    if(which > 1 && self->dev->image->height > self->dev->image->width)
-      g->current_aspect = 1.0/g->aspect_ratios[which];
-    else
-      g->current_aspect = g->aspect_ratios[which];
+
+    g->current_aspect = g->aspect_ratios[which];
+
     apply_box_aspect(self, 5);
-    dt_control_queue_draw_all();
+    dt_control_queue_redraw_center();
     dt_iop_request_focus(self);
   }
 }
@@ -683,7 +666,7 @@ angle_callback (GtkDarktableSlider *slider, dt_iop_module_t *self)
   if(self->dt->gui->reset) return;
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)self->params;
-  p->angle = dtgtk_slider_get_value(slider);
+  p->angle = - dtgtk_slider_get_value(slider);
   commit_box (self, g, p);
 }
 
@@ -708,18 +691,41 @@ keystone_callback_v (GtkWidget *widget, dt_iop_module_t *self)
   commit_box (self, g, p);
 }
 
+void gui_reset(struct dt_iop_module_t *self)
+{
+  /* reset aspect preset to default */
+  dt_conf_set_int("plugins/darkroom/clipping/aspect_preset", 1);
+
+}
+
 void gui_update(struct dt_iop_module_t *self)
 {
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)self->params;
-  dtgtk_slider_set_value(g->scale5, p->angle);
+
+  /* recalc aspect ratios for image */
+  _iop_clipping_update_ratios(self);
+
+  /* update ui elements */
+  dtgtk_slider_set_value(g->scale5, -p->angle);
   dtgtk_slider_set_value(g->keystone_h, p->k_h);
   dtgtk_slider_set_value(g->keystone_v, p->k_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->hflip), p->cw < 0);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->vflip), p->ch < 0);
+  
   int act = dt_conf_get_int("plugins/darkroom/clipping/aspect_preset");
-  if(act < -1 || act >= NUM_RATIOS) act = 0;
-  gtk_combo_box_set_active(GTK_COMBO_BOX(g->aspect_presets), act);
+  if (act < -1 || act >= NUM_RATIOS) 
+    act = 0;
+
+
+
+  /* special handling the combobox when current act is already selected
+     callback is not called, let do it our self then..
+   */
+  if (gtk_combo_box_get_active(GTK_COMBO_BOX(g->aspect_presets)) == act)
+    aspect_presets_changed(GTK_COMBO_BOX(g->aspect_presets), self);
+  else
+    gtk_combo_box_set_active(GTK_COMBO_BOX(g->aspect_presets), act);
 
   // reset gui draw box to what we have in the parameters:
   g->applied = 1;
@@ -727,6 +733,7 @@ void gui_update(struct dt_iop_module_t *self)
   g->clip_w = p->cw - p->cx;
   g->clip_y = p->cy;
   g->clip_h = p->ch - p->cy;
+
 }
 
 void init(dt_iop_module_t *module)
@@ -737,7 +744,7 @@ void init(dt_iop_module_t *module)
   module->default_enabled = 0;
   module->params_size = sizeof(dt_iop_clipping_params_t);
   module->gui_data = NULL;
-  module->priority = 391; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 392; // module order created by iop_dependencies.py, do not edit!
 }
 
 void cleanup(dt_iop_module_t *module)
@@ -780,10 +787,10 @@ key_swap_callback(GtkAccelGroup *accel_group, GObject *acceleratable,
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   g->current_aspect = 1.0/g->current_aspect;
   apply_box_aspect(self, 5);
-  dt_control_queue_draw_all();
+  dt_control_queue_redraw_center();
 }
 
-static void key_commit_callback(GtkAccelGroup *accel_group,
+static gboolean key_commit_callback(GtkAccelGroup *accel_group,
                                 GObject *acceleratable,
                                 guint keyval, GdkModifierType modifier,
                                 gpointer data)
@@ -792,6 +799,7 @@ static void key_commit_callback(GtkAccelGroup *accel_group,
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   dt_iop_clipping_params_t   *p = (dt_iop_clipping_params_t   *)self->params;
   commit_box(self, g, p);
+  return TRUE;
 }
 
 static void
@@ -800,11 +808,6 @@ aspect_flip(GtkWidget *button, dt_iop_module_t *self)
   key_swap_callback(NULL, NULL, 0, 0, self);
 }
 
-// Golden number (1+sqrt(5))/2
-#define PHI      1.61803398874989479F
-// 1/PHI
-#define INVPHI   0.61803398874989479F
-
 #define GUIDE_NONE 0
 #define GUIDE_GRID 1
 #define GUIDE_THIRD 2
@@ -846,14 +849,14 @@ guides_presets_changed (GtkComboBox *combo, dt_iop_module_t *self)
   }
 
   dt_iop_request_focus(self);
-  dt_control_queue_draw_all();
+  dt_control_queue_redraw_center();
 }
 
 static void
 guides_button_changed (GtkComboBox *combo, dt_iop_module_t *self)
 {
   // redraw guides
-  dt_control_queue_draw_all();
+  dt_control_queue_redraw_center();
 }
 
 void gui_init(struct dt_iop_module_t *self)
@@ -870,6 +873,7 @@ void gui_init(struct dt_iop_module_t *self)
   g->cropping = 0;
   g->straightening = 0;
   g->applied = 1;
+  g->center_lock = 0;
 
   self->widget = gtk_table_new(10, 6, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(self->widget), DT_GUI_IOP_MODULE_CONTROL_SPACING);
@@ -923,17 +927,20 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("1:2"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("3:2"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("4:3"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("5:4"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("square"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("DIN"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("16:9"));
+  
   dt_gui_key_accel_block_on_focus(gtk_bin_get_child(GTK_BIN(g->aspect_presets)));
+
   int act = dt_conf_get_int("plugins/darkroom/clipping/aspect_preset");
-  if(act < 0 || act >= 9) act = 0;
+  if(act < 0 || act >= NUM_RATIOS) act = 0;
   gtk_combo_box_set_active(GTK_COMBO_BOX(g->aspect_presets), act);
   g_signal_connect (G_OBJECT (g->aspect_presets), "changed",
                     G_CALLBACK (aspect_presets_changed), self);
   g_signal_connect (G_OBJECT (gtk_bin_get_child(GTK_BIN(g->aspect_presets))), "activate",
-                    G_CALLBACK (aspect_free_activated), self);
+		    G_CALLBACK (aspect_free_activated), self);
   g_object_set(G_OBJECT(g->aspect_presets), "tooltip-text", _("set the aspect ratio (w:h)\npress ctrl-x to swap sides"), (char *)NULL);
 
   GtkBox *hbox = GTK_BOX(gtk_hbox_new(FALSE, 5));
@@ -1020,24 +1027,37 @@ void gui_init(struct dt_iop_module_t *self)
 
   /*-------------------------------------------*/
 
+  _iop_clipping_update_ratios(self);
+
+  /* set default aspect ratio */
+  g->current_aspect = g->aspect_ratios[act];
+}
+
+void _iop_clipping_update_ratios(dt_iop_module_t *self)
+{
+  dt_iop_clipping_gui_data_t *g = self->gui_data;
+
   g->aspect_ratios[0] = -1;
-  g->aspect_ratios[1] = self->dev->image->width/(float)self->dev->image->height;
-  if(g->aspect_ratios[1] < 1.0)
-    g->aspect_ratios[1] = 1.0 / g->aspect_ratios[1];
-  g->aspect_ratios[2] = 1.6280;
+  g->aspect_ratios[1] = self->dev->image_storage.width / (float)self->dev->image_storage.height;
+  g->aspect_ratios[2] = PHI;
   g->aspect_ratios[3] = 2.0/1.0;
   g->aspect_ratios[4] = 3.0/2.0;
   g->aspect_ratios[5] = 4.0/3.0;
-  g->aspect_ratios[6] = 1.0;
-  g->aspect_ratios[7] = sqrtf(2.0);
-  g->aspect_ratios[8] = 16.0f/9.0f;
+  g->aspect_ratios[6] = 5.0f/4.0f;
+  g->aspect_ratios[7] = 1.0;
+  g->aspect_ratios[8] = sqrtf(2.0);
+  g->aspect_ratios[9] = 16.0f/9.0f;
+
   // if adding new presets, make sure to change this as well:
-  assert(NUM_RATIOS == 9);
+  assert(NUM_RATIOS == 10);
+
+  /* swap default fixed ratios for portraits */
+  if (g->aspect_ratios[1] < 1.0)
+  {
+    for (int k=2; k<NUM_RATIOS; k++)
+      g->aspect_ratios[k] = 1.0 / g->aspect_ratios[k];
+  }
 
-  if(act> 0 && self->dev->image->height > self->dev->image->width)
-    g->current_aspect = 1.0/g->aspect_ratios[act];
-  else
-    g->current_aspect = g->aspect_ratios[act];
 }
 
 void gui_cleanup(struct dt_iop_module_t *self)
@@ -1252,22 +1272,24 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
 
   double dashes = 5.0/zoom_scale;
 
-  // draw cropping window handles:
+  // draw cropping window
   float pzx, pzy;
   dt_dev_get_pointer_zoom_pos(dev, pointerx, pointery, &pzx, &pzy);
   pzx += 0.5f;
   pzy += 0.5f;
   cairo_set_dash (cr, &dashes, 0, 0);
-  if(g->applied)
-    cairo_set_source_rgba(cr, .2, .2, .2, .8);
-  else
-    cairo_set_source_rgba(cr, .3, .3, .3, .5);
+  cairo_set_source_rgba(cr, .2, .2, .2, .8);
   cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
   cairo_rectangle (cr, -1, -1, wd+2, ht+2);
   cairo_rectangle (cr, g->clip_x*wd, g->clip_y*ht, g->clip_w*wd, g->clip_h*ht);
   cairo_fill (cr);
 
-  cairo_stroke (cr);
+  if(g->clip_x > .0f || g->clip_y > .0f || g->clip_w < 1.0f || g->clip_h < 1.0f)
+  {
+    cairo_rectangle (cr, g->clip_x*wd, g->clip_y*ht, g->clip_w*wd, g->clip_h*ht);
+    cairo_set_source_rgb(cr, .7, .7, .7);
+    cairo_stroke (cr);
+  }
 
   // draw crop area guides
   float left, top, right, bottom, xThird, yThird;
@@ -1280,12 +1302,14 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   xThird = cwidth  / 3;
   yThird = cheight / 3;
 
-  // save context and draw guides
+  // save context
   cairo_save(cr);
   cairo_rectangle (cr, left, top, cwidth, cheight);
   cairo_clip(cr);
   cairo_set_line_width(cr, 1.0/zoom_scale);
   cairo_set_source_rgb(cr, .8, .8, .8);
+ 
+  // draw guides
   cairo_set_dash(cr, &dashes, 1, 0);
 
   int which = gtk_combo_box_get_active(g->guide_lines);
@@ -1425,7 +1449,7 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, int which)
     // second mouse button, straighten activated:
     g->straightening = 1;
     dt_control_change_cursor(GDK_CROSSHAIR);
-    dt_control_gui_queue_draw();
+    dt_control_queue_redraw_center();
   }
   else if(darktable.control->button_down && darktable.control->button_down_which == 1)
   {
@@ -1454,32 +1478,63 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, int which)
 
       if(grab == 15)
       {
+	/* moving the crop window */
         g->clip_x = fminf(1.0 - g->clip_w, fmaxf(0.0, g->handle_x + pzx - bzx));
         g->clip_y = fminf(1.0 - g->clip_h, fmaxf(0.0, g->handle_y + pzy - bzy));
       }
       else
       {
-        if(grab & 1)
-        {
-          const float old_clip_x = g->clip_x;
-          g->clip_x = fmaxf(0.0, pzx - g->handle_x);
-          g->clip_w = fmaxf(0.1, old_clip_x + g->clip_w - g->clip_x);
-        }
-        if(grab & 2)
-        {
-          const float old_clip_y = g->clip_y;
-          g->clip_y = fmaxf(0.0, pzy - g->handle_y);
-          g->clip_h = fmaxf(0.1, old_clip_y + g->clip_h - g->clip_y);
-        }
-        if(grab & 4) g->clip_w = fmaxf(0.1, fminf(1.0, pzx - g->clip_x - g->handle_x));
-        if(grab & 8) g->clip_h = fmaxf(0.1, fminf(1.0, pzy - g->clip_y - g->handle_y));
-      }
+	/* changing the crop window */
+	if (g->center_lock)
+	{
+	  /* the center is locked, scale crop radial with locked ratio */
+	  gboolean flag = FALSE;
+	  float length = 0.0;
+	  float xx = 0.0;
+	  float yy = 0.0;
 
-      if(g->clip_x + g->clip_w > 1.0) g->clip_w = 1.0 - g->clip_x;
-      if(g->clip_y + g->clip_h > 1.0) g->clip_h = 1.0 - g->clip_y;
+	  if (grab & 1 || grab & 4) 
+	    xx = (grab & 1) ? (pzx-bzx) : (bzx-pzx);
+	  if (grab & 2 || grab & 8)
+	    yy = (grab & 2) ? (pzy-bzy) : (bzy-pzy);
+	  
+	  length = (fabs(xx) > fabs(yy)) ? xx : yy;
+
+	  if ((g->prev_clip_w - (length+length)) < 0.1 ||
+	      (g->prev_clip_h - (length+length)) < 0.1)
+	    flag = TRUE;
+
+	  g->clip_x = flag ? g->clip_x : g->prev_clip_x + length;
+	  g->clip_y = flag ? g->clip_y : g->prev_clip_y + length;
+	  g->clip_w = fmax(0.1, g->prev_clip_w - (length+length));
+	  g->clip_h = fmax(0.1, g->prev_clip_h - (length+length));
+	 	  
+	}
+	else
+	{
+
+	  if(grab & 1)
+	  {
+	    const float old_clip_x = g->clip_x;
+	    g->clip_x = fmaxf(0.0, pzx - g->handle_x);
+	    g->clip_w = fmaxf(0.1, old_clip_x + g->clip_w - g->clip_x);
+	  }
+	  if(grab & 2)
+          {
+	    const float old_clip_y = g->clip_y;
+	    g->clip_y = fmaxf(0.0, pzy - g->handle_y);
+	    g->clip_h = fmaxf(0.1, old_clip_y + g->clip_h - g->clip_y);
+	  }
+	  if(grab & 4) g->clip_w = fmaxf(0.1, fminf(1.0, pzx - g->clip_x - g->handle_x));
+	  if(grab & 8) g->clip_h = fmaxf(0.1, fminf(1.0, pzy - g->clip_y - g->handle_y));
+	}
+
+	if(g->clip_x + g->clip_w > 1.0) g->clip_w = 1.0 - g->clip_x;
+	if(g->clip_y + g->clip_h > 1.0) g->clip_h = 1.0 - g->clip_y;
+      }
       apply_box_aspect(self, grab);
     }
-    dt_control_gui_queue_draw();
+    dt_control_queue_redraw_center();
     return 1;
   }
   else if (grab)
@@ -1497,14 +1552,14 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, int which)
       else if(grab == 12) dt_control_change_cursor(GDK_BOTTOM_RIGHT_CORNER);
       else if(grab == 9)  dt_control_change_cursor(GDK_BOTTOM_LEFT_CORNER);
     }
-    dt_control_gui_queue_draw();
+    dt_control_queue_redraw_center();
   }
   else
   {
     // somewhere besides borders. maybe rotate?
     if(old_grab != grab) dt_control_change_cursor(GDK_FLEUR);
     g->straightening = g->cropping = 0;
-    dt_control_gui_queue_draw();
+    dt_control_queue_redraw_center();
   }
   old_grab = grab;
   return 0;
@@ -1539,7 +1594,7 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   if(g->straightening)
   {
-    float dx = x - darktable.control->button_x, dy = y - darktable.control->button_y;
+    float dx = x - g->button_down_x, dy = y - g->button_down_y;
     if(dx < 0)
     {
       dx = -dx;
@@ -1555,10 +1610,12 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
     if(a < -180.0) a += 360.0;
     if(a >  180.0) a -= 360.0;
     if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
-    dtgtk_slider_set_value(g->scale5, a);
+    dtgtk_slider_set_value(g->scale5, -a);
     dt_control_change_cursor(GDK_LEFT_PTR);
   }
-  g->straightening = g->cropping = 0;
+
+  /* reset internal ui states*/
+  g->center_lock = g->straightening = g->cropping = 0;
   return 1;
 }
 
@@ -1566,15 +1623,32 @@ int button_pressed(struct dt_iop_module_t *self, double x, double y, int which,
 {
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   dt_iop_clipping_params_t   *p = (dt_iop_clipping_params_t   *)self->params;
-  if(which == 1 && darktable.control->button_type == GDK_2BUTTON_PRESS)
+  // avoid unexpected back to lt mode:
+  if(type == GDK_2BUTTON_PRESS && which == 1)
   {
-    commit_box(self, g, p);
+    dt_iop_request_focus(NULL);
     return 1;
   }
-  else if(which == 3 || which == 1)
+  if(which == 3 || which == 1)
   {
+    if (self->off) 
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
+
+    g->button_down_x = x;
+    g->button_down_y = y;
     dt_dev_get_pointer_zoom_pos(self->dev, x, y, &g->button_down_zoom_x, &g->button_down_zoom_y);
     g->button_down_angle = p->angle;
+
+    /* update prev clip box with current */
+    g->prev_clip_x = g->clip_x;
+    g->prev_clip_y = g->clip_y;
+    g->prev_clip_w = g->clip_w;
+    g->prev_clip_h = g->clip_h;
+
+    /* if shift is pressed, then lock crop on center */
+    if ((state&GDK_SHIFT_MASK) == GDK_SHIFT_MASK)
+      g->center_lock = 1;
+
     return 1;
   }
   else return 0;
