@@ -19,13 +19,8 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <stdlib.h>
-#include <math.h>
-#include <assert.h>
-#include <string.h>
-#include <gdk/gdkkeysyms.h>
-#include "iop/colorout.h"
 #include "develop/develop.h"
+#include "iop/colorout.h"
 #include "control/control.h"
 #include "control/conf.h"
 #include "gui/accelerators.h"
@@ -33,7 +28,13 @@
 #include "common/colorspaces.h"
 #include "common/opencl.h"
 #include "dtgtk/resetlabel.h"
+
 #include <xmmintrin.h>
+#include <stdlib.h>
+#include <math.h>
+#include <assert.h>
+#include <string.h>
+#include <gdk/gdkkeysyms.h>
 
 #define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
 
@@ -60,7 +61,7 @@ flags ()
 }
 
 
-static void key_softproof_callback(GtkAccelGroup *accel_group,
+static gboolean key_softproof_callback(GtkAccelGroup *accel_group,
                                    GObject *acceleratable,
                                    guint keyval, GdkModifierType modifier,
                                    gpointer data)
@@ -80,7 +81,8 @@ static void key_softproof_callback(GtkAccelGroup *accel_group,
   }
 
   dt_dev_add_history_item(darktable.develop, self, TRUE);
-  dt_control_queue_draw_all();
+  dt_control_queue_redraw_center();
+  return TRUE;
 }
 
 int
@@ -651,7 +653,7 @@ void init(dt_iop_module_t *module)
   module->default_params = malloc(sizeof(dt_iop_colorout_params_t));
   module->params_size = sizeof(dt_iop_colorout_params_t);
   module->gui_data = NULL;
-  module->priority = 782; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 803; // module order created by iop_dependencies.py, do not edit!
   module->hide_enable_button = 1;
   dt_iop_colorout_params_t tmp = (dt_iop_colorout_params_t)
     {"sRGB", "X profile", DT_INTENT_PERCEPTUAL, DT_INTENT_PERCEPTUAL,
