@@ -85,7 +85,7 @@ static void
 request_pick_toggled(GtkToggleButton *togglebutton, dt_iop_module_t *self)
 {
   self->request_color_pick = gtk_toggle_button_get_active(togglebutton);
-  if(darktable.gui->reset || self->dev->image->filters) return;
+  if(darktable.gui->reset || self->dev->image_storage.filters) return;
   if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
   dt_iop_request_focus(self);
 }
@@ -96,7 +96,7 @@ expose (GtkWidget *widget, GdkEventExpose *event, dt_iop_module_t *self)
   if(darktable.gui->reset) return FALSE;
   if(self->picked_color_max[0] < 0) return FALSE;
   if(!self->request_color_pick) return FALSE;
-  if(self->dev->image->filters) return FALSE;
+  if(self->dev->image_storage.filters) return FALSE;
   dt_iop_invert_gui_data_t *g = (dt_iop_invert_gui_data_t *)self->gui_data;
   dt_iop_invert_params_t *p = (dt_iop_invert_params_t *)self->params;
 
@@ -130,12 +130,12 @@ colorpick_button_callback(GtkButton *button, GtkColorSelectionDialog *csd)
 static void
 colorpicker_callback (GtkDarktableButton *button, dt_iop_module_t *self)
 {
-  if(self->dt->gui->reset || self->dev->image->filters) return;
+  if(self->dt->gui->reset || self->dev->image_storage.filters) return;
   dt_iop_invert_gui_data_t *g = (dt_iop_invert_gui_data_t *)self->gui_data;
   dt_iop_invert_params_t *p = (dt_iop_invert_params_t *)self->params;
 
   GtkColorSelectionDialog  *csd = GTK_COLOR_SELECTION_DIALOG(gtk_color_selection_dialog_new(_("select color of film material")));
-  gtk_window_set_transient_for(GTK_WINDOW(csd), GTK_WINDOW(darktable.gui->widgets.main_window));
+  gtk_window_set_transient_for(GTK_WINDOW(csd), GTK_WINDOW(dt_ui_main_window(darktable.gui->ui)));
   g_signal_connect (G_OBJECT (csd->ok_button), "clicked",
                     G_CALLBACK (colorpick_button_callback), csd);
   g_signal_connect (G_OBJECT (csd->cancel_button), "clicked",
@@ -175,9 +175,9 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
     // do nothing
 //   }
 
-  const int filters = dt_image_flipped_filter(self->dev->image);
+  const int filters = dt_image_flipped_filter(&piece->pipe->image);
 
-  if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && self->dev->image->bpp != 4)
+  if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && piece->pipe->image.bpp != 4)
   {
     // doesn't work and isn't used.
 //     uint16_t min = -1, max = 0, res[3] = {0,0,0};
@@ -203,7 +203,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 //     for(int k=0; k<3; k++)
 //       piece->pipe->processed_maximum[k] = res[k];
   }
-  else if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && self->dev->image->bpp == 4)
+  else if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && piece->pipe->image.bpp == 4)
   {
     // doesn't work and isn't used.
 //     float min, max;
@@ -251,8 +251,8 @@ void reload_defaults(dt_iop_module_t *self)
   memcpy(self->default_params, &tmp, sizeof(dt_iop_invert_params_t));
 
   // can't be switched on for raw and hdr images:
-  if(self->dev->image->flags & DT_IMAGE_RAW) self->hide_enable_button = 1;
-  else                                       self->hide_enable_button = 0;
+  if(dt_image_is_raw(&self->dev->image_storage)) self->hide_enable_button = 1;
+  else self->hide_enable_button = 0;
 }
 
 void init(dt_iop_module_t *module)
@@ -263,7 +263,7 @@ void init(dt_iop_module_t *module)
   module->default_enabled = 0;
   module->params_size = sizeof(dt_iop_invert_params_t);
   module->gui_data = NULL;
-  module->priority = 21; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 19; // module order created by iop_dependencies.py, do not edit!
 }
 
 void cleanup(dt_iop_module_t *module)
@@ -299,7 +299,7 @@ void gui_update(dt_iop_module_t *self)
   dt_iop_invert_params_t *p = (dt_iop_invert_params_t *)self->params;
 
   // FIXME: double clicking the reset label twice allows this to be enabled for raw files ...
-  if(self->dev->image->filters)
+  if(self->dev->image_storage.filters)
   {
     gtk_widget_set_visible(GTK_WIDGET(g->pickerbuttons), FALSE);
     dtgtk_reset_label_set_text(g->label, _("this doesn't work for raw/hdr images."));
