@@ -130,7 +130,7 @@ void connect_key_accels(dt_iop_module_t *self)
 int
 output_bpp(dt_iop_module_t *module, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
-  if(pipe->type != DT_DEV_PIXELPIPE_PREVIEW && module->dev->image->filters) return sizeof(float);
+  if(pipe->type != DT_DEV_PIXELPIPE_PREVIEW && (pipe->image.flags & DT_IMAGE_RAW)) return sizeof(float);
   else return 4*sizeof(float);
 }
 
@@ -198,9 +198,9 @@ FC(const int row, const int col, const unsigned int filters)
 
 void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *ivoid, void *ovoid, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
-  const int filters = dt_image_flipped_filter(self->dev->image);
+  const int filters = dt_image_flipped_filter(&piece->pipe->image);
   dt_iop_temperature_data_t *d = (dt_iop_temperature_data_t *)piece->data;
-  if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && self->dev->image->bpp != 4)
+  if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && piece->pipe->image.bpp != 4)
   {
     const float coeffsi[3] = {d->coeffs[0]/65535.0f, d->coeffs[1]/65535.0f, d->coeffs[2]/65535.0f};
 #ifdef _OPENMP
@@ -233,7 +233,7 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
     }
     _mm_sfence();
   }
-  else if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && self->dev->image->bpp == 4)
+  else if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && piece->pipe->image.bpp == 4)
   {
 #ifdef _OPENMP
     #pragma omp parallel for default(none) shared(roi_out, ivoid, ovoid, d) schedule(static)
@@ -272,7 +272,7 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   dt_iop_temperature_global_data_t *gd = (dt_iop_temperature_global_data_t *)self->data;
 
   const int devid = piece->pipe->devid;
-  const int filters = dt_image_flipped_filter(self->dev->image);
+  const int filters = dt_image_flipped_filter(&piece->pipe->image);
   const int ui = ((piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW) && filters);
   float coeffs[3] = {d->coeffs[0], d->coeffs[1], d->coeffs[2]};
   if(ui) for(int k=0; k<3; k++) coeffs[k] /= 65535.0f;
@@ -349,6 +349,7 @@ void gui_update (struct dt_iop_module_t *self)
   self->request_color_pick = 0;
   self->color_picker_box[0] = self->color_picker_box[1] = .25f;
   self->color_picker_box[2] = self->color_picker_box[3] = .75f;
+  self->color_picker_point[0] = self->color_picker_point[1] = 0.5f;
   dt_iop_temperature_gui_data_t *g = (dt_iop_temperature_gui_data_t *)self->gui_data;
   dt_iop_temperature_params_t *p  = (dt_iop_temperature_params_t *)module->params;
   dt_iop_temperature_params_t *fp = (dt_iop_temperature_params_t *)module->factory_params;
@@ -372,7 +373,7 @@ void gui_update (struct dt_iop_module_t *self)
 void reload_defaults(dt_iop_module_t *module)
 {
   // raw images need wb (to convert from uint16_t to float):
-  if(module->dev->image->flags & DT_IMAGE_RAW)
+  if(dt_image_is_raw(&module->dev->image_storage))
   {
     module->default_enabled = 1;
     module->hide_enable_button = 1;
@@ -387,14 +388,15 @@ void reload_defaults(dt_iop_module_t *module)
   char filename[1024];
   int ret=0;
   /* check if file is raw / hdr */
-  if (! (module->dev->image->flags & DT_IMAGE_LDR))
+  if(dt_image_is_raw(&module->dev->image_storage))
   {
-    dt_image_full_path(module->dev->image->id, filename, 1024);
+    dt_image_full_path(module->dev->image_storage.id, filename, 1024);
     libraw_data_t *raw = libraw_init(0);
   
     ret = libraw_open_file(raw, filename);
     if(!ret)
     {
+      module->default_enabled = 1;
       for(int k=0; k<3; k++) tmp.coeffs[k] = raw->color.cam_mul[k];
       if(tmp.coeffs[0] <= 0.0)
       {
@@ -432,7 +434,7 @@ void init (dt_iop_module_t *module)
 {
   module->params = malloc(sizeof(dt_iop_temperature_params_t));
   module->default_params = malloc(sizeof(dt_iop_temperature_params_t));
-  module->priority = 43; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 39; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_temperature_params_t);
   module->gui_data = NULL;
 }
@@ -552,8 +554,8 @@ void gui_init (struct dt_iop_module_t *self)
   const char *wb_name = NULL;
   char makermodel[1024];
   char *model = makermodel;
-  dt_colorspaces_get_makermodel_split(makermodel, 1024, &model, self->dev->image->exif_maker, self->dev->image->exif_model);
-  if(!dt_image_is_ldr(self->dev->image)) for(int i=0; i<wb_preset_count; i++)
+  dt_colorspaces_get_makermodel_split(makermodel, 1024, &model, self->dev->image_storage.exif_maker, self->dev->image_storage.exif_model);
+  if(!dt_image_is_ldr(&self->dev->image_storage)) for(int i=0; i<wb_preset_count; i++)
     {
       if(g->preset_cnt >= 50) break;
       if(!strcmp(wb_preset[i].make,  makermodel) &&
@@ -572,6 +574,7 @@ void gui_init (struct dt_iop_module_t *self)
   g->finetune = GTK_SPIN_BUTTON(gtk_spin_button_new_with_range(-9, 9, 1));
   gtk_spin_button_set_value (g->finetune, 0);
   gtk_spin_button_set_digits(g->finetune, 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(g->finetune), FALSE);
   gtk_box_pack_start(hbox, GTK_WIDGET(g->finetune), FALSE, FALSE, 0);
   g_object_set(G_OBJECT(g->finetune), "tooltip-text", _("fine tune white balance preset"), (char *)NULL);
 
@@ -702,6 +705,11 @@ apply_preset(dt_iop_module_t *self)
       for(int k=0; k<3; k++) p->coeffs[k] = fp->coeffs[k];
       dt_iop_request_focus(self);
       self->request_color_pick = 1;
+
+      /* set the area sample size*/
+      if (self->request_color_pick)
+	dt_lib_colorpicker_set_area(darktable.lib, 0.99);
+
       break;
     case 2: // passthrough mode, raw data
       for(int k=0; k<3; k++) p->coeffs[k] = 1.0;
@@ -711,7 +719,7 @@ apply_preset(dt_iop_module_t *self)
       {
         char makermodel[1024];
         char *model = makermodel;
-        dt_colorspaces_get_makermodel_split(makermodel, 1024, &model, self->dev->image->exif_maker, self->dev->image->exif_model);
+        dt_colorspaces_get_makermodel_split(makermodel, 1024, &model, self->dev->image_storage.exif_maker, self->dev->image_storage.exif_model);
         if(!strcmp(wb_preset[i].make,  makermodel) &&
             !strcmp(wb_preset[i].model, model) && wb_preset[i].tuning == tune)
         {
