@@ -20,9 +20,22 @@
 
 #include "common/image.h"
 #include <inttypes.h>
+#include <gui/gtk.h>
 #include <gmodule.h>
 #include <cairo.h>
 #include <sqlite3.h>
+
+/** avilable views flags, a view should return it's type and
+    is also used in modules flags available in src/libs to
+    control which view the module should be available in also
+    which placement in the panels the module have.
+*/
+enum dt_view_type_flags_t {
+  DT_VIEW_LIGHTTABLE = 1,
+  DT_VIEW_DARKROOM = 2,
+  DT_VIEW_TETHERING = 4,
+};
+
 /**
  * main dt view module (as lighttable or darkroom)
  */
@@ -40,6 +53,7 @@ typedef struct dt_view_t
   float vscroll_size, vscroll_viewport_size, vscroll_pos;
   float hscroll_size, hscroll_viewport_size, hscroll_pos;
   const char *(*name)     (struct dt_view_t *self); // get translatable name
+  uint32_t (*view)        (struct dt_view_t *self); // get the view type
   void (*init)            (struct dt_view_t *self); // init *data
   void (*cleanup)         (struct dt_view_t *self); // cleanup *data
   void (*expose)          (struct dt_view_t *self, cairo_t *cr, int32_t width, int32_t height, int32_t pointerx, int32_t pointery); // expose the module (gtk callback)
@@ -81,7 +95,16 @@ typedef enum dt_view_image_over_t
 dt_view_image_over_t;
 
 /** expose an image, set image over flags. */
-void dt_view_image_expose(dt_image_t *img, dt_view_image_over_t *image_over, int32_t index, cairo_t *cr, int32_t width, int32_t height, int32_t zoom, int32_t px, int32_t py);
+void
+dt_view_image_expose(
+    dt_view_image_over_t *image_over,
+    uint32_t index,
+    cairo_t *cr,
+    int32_t width,
+    int32_t height,
+    int32_t zoom,
+    int32_t px,
+    int32_t py);
 
 /** Set the selection bit to a given value for the specified image */
 void dt_view_set_selection(int imgid, int value);
@@ -95,14 +118,8 @@ void dt_view_toggle_selection(int imgid);
  */
 typedef struct dt_view_manager_t
 {
-  dt_view_t film_strip;
   dt_view_t view[DT_VIEW_MAX_MODULES];
   int32_t current_view, num_views;
-  int32_t film_strip_on;
-  float film_strip_size;
-  int32_t film_strip_dragging, film_strip_scroll_to, film_strip_active_image;
-  void (*film_strip_activated)(const int imgid, void *data);
-  void *film_strip_data;
 
   /* reusable db statements 
    * TODO: reconsider creating a common/database helper API
@@ -121,6 +138,57 @@ typedef struct dt_view_manager_t
     sqlite3_stmt *get_color;
 
   } statements;
+
+
+  /*
+   * Proxy
+   */
+  struct {
+
+    /* view toolbox proxy object */
+    struct {
+      struct dt_lib_module_t *module;
+      void (*add)(struct dt_lib_module_t *,GtkWidget *);
+    } view_toolbox;
+
+    /* module toolbox proxy object */
+    struct {
+      struct dt_lib_module_t *module;
+      void (*add)(struct dt_lib_module_t *,GtkWidget *);
+    } module_toolbox;
+
+    /* module collection proxy object */
+    struct {
+      struct dt_lib_module_t *module;
+      void (*update)(struct dt_lib_module_t *);
+    } module_collect;
+
+    /* filmstrip proxy object */
+    struct {
+      struct dt_lib_module_t *module;
+      void (*scroll_to_image)(struct dt_lib_module_t *, gint imgid, gboolean activate);
+      int32_t (*activated_image)(struct dt_lib_module_t *);
+    } filmstrip;
+
+    /* lighttable view proxy object */
+    struct {
+      struct dt_lib_module_t *module;
+      void (*set_zoom)(struct dt_lib_module_t *module, gint zoom);
+    } lighttable;
+
+    /* tethering view proxy object */
+    struct {
+      struct dt_view_t *view;
+      uint32_t (*get_film_id)(const dt_view_t *view);
+      const char *(*get_session_filename)(const dt_view_t *view, const char *filename);
+      const char *(*get_session_path)(const dt_view_t *view);
+      const char *(*get_job_code)(const dt_view_t *view);
+      void (*set_job_code)(const dt_view_t *view, const char *name);
+    } tethering;
+
+  } proxy;
+
+
 }
 dt_view_manager_t;
 
@@ -149,6 +217,12 @@ void dt_view_manager_configure       (dt_view_manager_t *vm, int width, int heig
 void dt_view_manager_scrolled        (dt_view_manager_t *vm, double x, double y, int up, int state);
 void dt_view_manager_border_scrolled (dt_view_manager_t *vm, double x, double y, int which, int up);
 
+/** add widget to the current view toolbox */
+void dt_view_manager_view_toolbox_add(dt_view_manager_t *vm,GtkWidget *tool);
+
+/** add widget to the current module toolbox */
+void dt_view_manager_module_toolbox_add(dt_view_manager_t *vm,GtkWidget *tool);
+
 /** load module to view managers list, if still space. return slot number on success. */
 int dt_view_manager_load_module(dt_view_manager_t *vm, const char *mod);
 /** load a view module */
@@ -157,19 +231,42 @@ int dt_view_load_module(dt_view_t *view, const char *module);
 void dt_view_unload_module(dt_view_t *view);
 /** set scrollbar positions, gui method. */
 void dt_view_set_scrollbar(dt_view_t *view, float hpos, float hsize, float hwinsize, float vpos, float vsize, float vwinsize);
-/** open up the film strip view, with given callback on image activation. */
-void dt_view_film_strip_open(dt_view_manager_t *vm, void (*activated)(const int, void*), void *data);
-/** close the film strip view. */
-void dt_view_film_strip_close(dt_view_manager_t *vm);
-/** toggles the film strip. */
-void dt_view_film_strip_toggle(dt_view_manager_t *vm, void (*activated)(const int imgid, void*), void *data);
-/** advise the film strip to scroll to imgid at next expose. */
-void dt_view_film_strip_scroll_to(dt_view_manager_t *vm, const int imgid);
-/** prefetch the next few images in film strip, from selected on. */
-void dt_view_film_strip_prefetch();
-/** Clears all selection and selects the given image as active image in film strip. */
-void dt_view_film_strip_set_active_image(dt_view_manager_t *vm,int iid);
-/** Gets the active image id in filmstrip */
-uint32_t dt_view_film_strip_get_active_image(dt_view_manager_t *vm);
+
+/*
+ * Tethering View PROXY
+ */
+/** get the current filmroll id for tethering session */
+int32_t dt_view_tethering_get_film_id(const dt_view_manager_t *vm);
+/** get the current session path for tethering session */
+const char *dt_view_tethering_get_session_path(const dt_view_manager_t *vm);
+/** get the current session filename for tethering session */
+const char *dt_view_tethering_get_session_filename(const dt_view_manager_t *vm, const char *filename);
+/** set the current jobcode for tethering session */
+void dt_view_tethering_set_job_code(const dt_view_manager_t *vm, const char *name);
+/** get the current jobcode for tethering session */
+const char *dt_view_tethering_get_job_code(const dt_view_manager_t *vm);
+
+/** update the collection module */
+void dt_view_collection_update(const dt_view_manager_t *vm);
+
+/*
+ * NEW filmstrip api
+ */
+/** scrolls filmstrip to the specified image */
+void dt_view_filmstrip_scroll_to_image(dt_view_manager_t *vm, const int imgid, gboolean activate);
+/** get the imageid from last filmstrip activate request */
+int32_t dt_view_filmstrip_get_activated_imgid(dt_view_manager_t *vm);
+
+/** sets the lighttable image in row zoom */
+void dt_view_lighttable_set_zoom(dt_view_manager_t *vm, gint zoom);
+
+/** set active image */
+void dt_view_filmstrip_set_active_image(dt_view_manager_t *vm,int iid);
+/** prefetch the next few images in film strip, from selected on. 
+    TODO: move to control ?
+*/
+void dt_view_filmstrip_prefetch();
+
+
 
 #endif
