@@ -1,11 +1,12 @@
 #!/bin/bash
 
-git log release-0.8..master | grep ^Author: | sed 's/ <.*//; s/^Author: //' | sort | uniq -c | sort -nr
+git log release-0.9.3..master | grep ^Author: | sed 's/ <.*//; s/^Author: //' | sort | uniq -c | sort -nr
 
 echo "are you sure these guys received proper credit in the about dialog?"
 read
 
-dt_decoration=$(git describe --tags $branch | sed 's,^release-,,;s,-,+,;s,-,~,;')
+# prefix rc with ~, so debian thinks its less than
+dt_decoration=$(git describe --tags $branch | sed 's,^release-,,;s,-,+,;s,-,~,;' | sed 's/rc/~rc/')
 git archive master --prefix=darktable-$dt_decoration/ -o darktable-$dt_decoration.tar
 
 mkdir -p tmp
